@@ -1,7 +1,5 @@
 /*
--*- linux-c -*-
    drbd_bitmap.c
-   Kernel module for 2.6.x Kernels
 
    This file is part of DRBD by Philipp Reisner and Lars Ellenberg.
 
@@ -28,6 +26,7 @@
 #include <linux/vmalloc.h>
 #include <linux/string.h>
 #include <linux/drbd.h>
+#include <asm/kmap_types.h>
 #include "drbd_int.h"
 
 /* OPAQUE outside this file!
@@ -40,13 +39,6 @@
  * Note that since find_first_bit returns int, at the current granularity of
  * the bitmap (4KB per byte), this implementation "only" supports up to
  * 1<<(32+12) == 16 TB...
- *
- * we will eventually change the implementation to not allways hold the full
- * bitmap in memory, but only some 'lru_cache' of the on disk bitmap.
-
- * THINK
- * I'm not yet sure whether this file should be bits only,
- * or wether I want it to do all the sector<->bit calculation in here.
  */
 
 /*
@@ -58,34 +50,17 @@
  *  We may be called with irq already disabled,
  *  so we need spin_lock_irqsave().
  *  And we need the kmap_atomic.
- * FIXME
- *  for performance reasons, when we _know_ we have irq disabled, we should
- *  probably introduce some _in_irq variants, so we know to only spin_lock().
- *
- * FIXME
- *  Actually you need to serialize all resize operations.
- *  but then, resize is a drbd state change, and it should be serialized
- *  already. Unfortunately it is not (yet), so two concurrent resizes, like
- *  attach storage (drbdsetup) and receive the peers size (drbd receiver)
- *  may eventually blow things up.
- * Therefore,
- *  you may only change the other members when holding
- *  the bm_change mutex _and_ the bm_lock.
- *  thus reading them holding either is safe.
- *  this is sort of overkill, but I rather do it right
- *  than have two resize operations interfere somewhen.
  */
 struct drbd_bitmap {
 	struct page **bm_pages;
 	spinlock_t bm_lock;
-	/* WARNING unsigned long bm_fo and friends:
+	/* WARNING unsigned long bm_*:
 	 * 32bit number of bit offset is just enough for 512 MB bitmap.
 	 * it will blow up if we make the bitmap bigger...
 	 * not that it makes much sense to have a bitmap that large,
 	 * rather change the granularity to 16k or 64k or something.
 	 * (that implies other problems, however...)
 	 */
-	unsigned long bm_fo;        /* next offset for drbd_bm_find_next */
 	unsigned long bm_set;       /* nr of set bits; THINK maybe atomic_t? */
 	unsigned long bm_bits;
 	size_t   bm_words;
@@ -104,10 +79,11 @@ struct drbd_bitmap {
 };
 
 /* definition of bits in bm_flags */
-#define BM_LOCKED 0
-#define BM_MD_IO_ERROR (BITS_PER_LONG-1) /* 31? 63? */
+#define BM_LOCKED       0
+#define BM_MD_IO_ERROR  1
+#define BM_P_VMALLOCED  2
 
-static inline int bm_is_locked(struct drbd_bitmap *b)
+static int bm_is_locked(struct drbd_bitmap *b)
 {
 	return test_bit(BM_LOCKED, &b->bm_flags);
 }
@@ -118,7 +94,7 @@ static void __bm_print_lock_info(struct drbd_conf *mdev, const char *func)
 	struct drbd_bitmap *b = mdev->bitmap;
 	if (!DRBD_ratelimit(5*HZ, 5))
 		return;
-	ERR("FIXME %s in %s, bitmap locked for '%s' by %s\n",
+	dev_err(DEV, "FIXME %s in %s, bitmap locked for '%s' by %s\n",
 	    current == mdev->receiver.task ? "receiver" :
 	    current == mdev->asender.task  ? "asender"  :
 	    current == mdev->worker.task   ? "worker"   : current->comm,
@@ -134,17 +110,17 @@ void drbd_bm_lock(struct drbd_conf *mdev, char *why)
 	int trylock_failed;
 
 	if (!b) {
-		ERR("FIXME no bitmap in drbd_bm_lock!?\n");
+		dev_err(DEV, "FIXME no bitmap in drbd_bm_lock!?\n");
 		return;
 	}
 
 	trylock_failed = down_trylock(&b->bm_change);
 
 	if (trylock_failed) {
-		DBG("%s going to '%s' but bitmap already locked for '%s' by %s\n",
+		dev_warn(DEV, "%s going to '%s' but bitmap already locked for '%s' by %s\n",
 		    current == mdev->receiver.task ? "receiver" :
 		    current == mdev->asender.task  ? "asender"  :
-		    current == mdev->worker.task   ? "worker"   : "?",
+		    current == mdev->worker.task   ? "worker"   : current->comm,
 		    why, b->bm_why ?: "?",
 		    b->bm_task == mdev->receiver.task ? "receiver" :
 		    b->bm_task == mdev->asender.task  ? "asender"  :
@@ -152,7 +128,7 @@ void drbd_bm_lock(struct drbd_conf *mdev, char *why)
 		down(&b->bm_change);
 	}
 	if (__test_and_set_bit(BM_LOCKED, &b->bm_flags))
-		ERR("FIXME bitmap already locked in bm_lock\n");
+		dev_err(DEV, "FIXME bitmap already locked in bm_lock\n");
 
 	b->bm_why  = why;
 	b->bm_task = current;
@@ -162,12 +138,12 @@ void drbd_bm_unlock(struct drbd_conf *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	if (!b) {
-		ERR("FIXME no bitmap in drbd_bm_unlock!?\n");
+		dev_err(DEV, "FIXME no bitmap in drbd_bm_unlock!?\n");
 		return;
 	}
 
 	if (!__test_and_clear_bit(BM_LOCKED, &mdev->bitmap->bm_flags))
-		ERR("FIXME bitmap not locked in bm_unlock\n");
+		dev_err(DEV, "FIXME bitmap not locked in bm_unlock\n");
 
 	b->bm_why  = NULL;
 	b->bm_task = NULL;
@@ -192,7 +168,7 @@ void drbd_bm_unlock(struct drbd_conf *mdev)
 #endif
 
 /* word offset to long pointer */
-STATIC unsigned long * bm_map_paddr(struct drbd_bitmap *b, unsigned long offset)
+STATIC unsigned long *__bm_map_paddr(struct drbd_bitmap *b, unsigned long offset, const enum km_type km)
 {
 	struct page *page;
 	unsigned long page_nr;
@@ -202,16 +178,26 @@ STATIC unsigned long * bm_map_paddr(struct drbd_bitmap *b, unsigned long offset)
 	BUG_ON(page_nr >= b->bm_number_of_pages);
 	page = b->bm_pages[page_nr];
 
-	return (unsigned long *) kmap_atomic(page, KM_IRQ1);
+	return (unsigned long *) kmap_atomic(page, km);
 }
 
-STATIC void bm_unmap(unsigned long *p_addr)
+static unsigned long * bm_map_paddr(struct drbd_bitmap *b, unsigned long offset)
 {
-	kunmap_atomic(p_addr, KM_IRQ1);
+	return __bm_map_paddr(b, offset, KM_IRQ1);
+}
+
+static void __bm_unmap(unsigned long *p_addr, const enum km_type km)
+{
+	kunmap_atomic(p_addr, km);
 };
 
+static void bm_unmap(unsigned long *p_addr)
+{
+	return __bm_unmap(p_addr, KM_IRQ1);
+}
+
 /* long word offset of _bitmap_ sector */
-#define S2W(s)	((s)<<(BM_EXT_SIZE_B-BM_BLOCK_SIZE_B-LN2_BPL))
+#define S2W(s)	((s)<<(BM_EXT_SHIFT-BM_BLOCK_SHIFT-LN2_BPL))
 /* word offset from start of bitmap to word number _in_page_
  * modulo longs per page
 #define MLPP(X) ((X) % (PAGE_SIZE/sizeof(long))
@@ -229,12 +215,6 @@ STATIC void bm_unmap(unsigned long *p_addr)
  * to be able to report device specific.
  */
 
-/* FIXME TODO sometimes I use "int offset" as index into the bitmap.
- * since we currently are LIMITED to (128<<11)-64-8 sectors of bitmap,
- * this is ok [as long as we dont run on a 24 bit arch :)].
- * But it is NOT strictly ok.
- */
-
 STATIC void bm_free_pages(struct page **pages, unsigned long number)
 {
 	unsigned long i;
@@ -253,15 +233,23 @@ STATIC void bm_free_pages(struct page **pages, unsigned long number)
 	}
 }
 
+STATIC void bm_vk_free(void *ptr, int v)
+{
+	if (v)
+		vfree(ptr);
+	else
+		kfree(ptr);
+}
+
 /*
  * "have" and "want" are NUMBER OF PAGES.
  */
-STATIC struct page **bm_realloc_pages(struct page **old_pages,
-				       unsigned long have,
-				       unsigned long want)
+STATIC struct page **bm_realloc_pages(struct drbd_bitmap *b, unsigned long want)
 {
-	struct page** new_pages, *page;
-	unsigned int i, bytes;
+	struct page **old_pages = b->bm_pages;
+	struct page **new_pages, *page;
+	unsigned int i, bytes, vmalloced = 0;
+	unsigned long have = b->bm_number_of_pages;
 
 	BUG_ON(have == 0 && old_pages != NULL);
 	BUG_ON(have != 0 && old_pages == NULL);
@@ -269,36 +257,29 @@ STATIC struct page **bm_realloc_pages(struct page **old_pages,
 	if (have == want)
 		return old_pages;
 
-	/* To use kmalloc here is ok, as long as we support 4TB at max...
-	 * otherwise this might become bigger than 128KB, which is
-	 * the maximum for kmalloc.
-	 *
-	 * no, it is not: on 64bit boxes, sizeof(void*) == 8,
-	 * 128MB bitmap @ 4K pages -> 256K of page pointers.
-	 * ==> use vmalloc for now again.
-	 * then again, we could do something like
-	 *   if (nr_pages > watermark) vmalloc else kmalloc :*> ...
-	 * or do cascading page arrays:
-	 *   one page for the page array of the page array,
-	 *   those pages for the real bitmap pages.
-	 *   there we could even add some optimization members,
-	 *   so we won't need to kmap_atomic in bm_find_next_bit just to see
-	 *   that the page has no bits set ...
-	 * or we can try a "huge" page ;-)
-	 */
-	bytes = sizeof(struct page*)*want;
-	new_pages = vmalloc(bytes);
-	if (!new_pages)
-		return NULL;
+	/* Trying kmalloc first, falling back to vmalloc.
+	 * GFP_KERNEL is ok, as this is done when a lower level disk is
+	 * "attached" to the drbd.  Context is receiver thread or cqueue
+	 * thread.  As we have no disk yet, we are not in the IO path,
+	 * not even the IO path of the peer. */
+	bytes = sizeof(struct page *)*want;
+	new_pages = kmalloc(bytes, GFP_KERNEL);
+	if (!new_pages) {
+		new_pages = vmalloc(bytes);
+		if (!new_pages)
+			return NULL;
+		vmalloced = 1;
+	}
 
 	memset(new_pages, 0, bytes);
 	if (want >= have) {
 		for (i = 0; i < have; i++)
 			new_pages[i] = old_pages[i];
 		for (; i < want; i++) {
-			if (!(page = alloc_page(GFP_HIGHUSER))) {
+			page = alloc_page(GFP_HIGHUSER);
+			if (!page) {
 				bm_free_pages(new_pages + have, i - have);
-				vfree(new_pages);
+				bm_vk_free(new_pages, vmalloced);
 				return NULL;
 			}
 			new_pages[i] = page;
@@ -311,6 +292,11 @@ STATIC struct page **bm_realloc_pages(struct page **old_pages,
 		*/
 	}
 
+	if (vmalloced)
+		set_bit(BM_P_VMALLOCED, &b->bm_flags);
+	else
+		clear_bit(BM_P_VMALLOCED, &b->bm_flags);
+
 	return new_pages;
 }
 
@@ -345,13 +331,8 @@ sector_t drbd_bm_capacity(struct drbd_conf *mdev)
 void drbd_bm_cleanup(struct drbd_conf *mdev)
 {
 	ERR_IF (!mdev->bitmap) return;
-	/* FIXME I think we should explicitly change the device size to zero
-	 * before this...
-	 *
-	WARN_ON(mdev->bitmap->bm);
-	 */
 	bm_free_pages(mdev->bitmap->bm_pages, mdev->bitmap->bm_number_of_pages);
-	vfree(mdev->bitmap->bm_pages);
+	bm_vk_free(mdev->bitmap->bm_pages, test_bit(BM_P_VMALLOCED, &mdev->bitmap->bm_flags));
 	kfree(mdev->bitmap);
 	mdev->bitmap = NULL;
 }
@@ -411,7 +392,7 @@ STATIC void bm_set_surplus(struct drbd_bitmap *b)
 	bm_unmap(p_addr);
 }
 
-STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endian)
+STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endian, const enum km_type km)
 {
 	unsigned long *p_addr, *bm, offset = 0;
 	unsigned long bits = 0;
@@ -419,7 +400,7 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 
 	while (offset < b->bm_words) {
 		i = do_now = min_t(size_t, b->bm_words-offset, LWPP);
-		p_addr = bm_map_paddr(b, offset);
+		p_addr = __bm_map_paddr(b, offset, km);
 		bm = p_addr + MLPP(offset);
 		while (i--) {
 			catch_oob_access_start();
@@ -430,21 +411,21 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 			bits += hweight_long(*bm++);
 			catch_oob_access_end();
 		}
-		bm_unmap(p_addr);
+		__bm_unmap(p_addr, km);
 		offset += do_now;
 	}
 
 	return bits;
 }
 
-static inline unsigned long bm_count_bits(struct drbd_bitmap *b)
+static unsigned long bm_count_bits(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 0);
+	return __bm_count_bits(b, 0, KM_IRQ1);
 }
 
-static inline unsigned long bm_count_bits_swap_endian(struct drbd_bitmap *b)
+static unsigned long bm_count_bits_swap_endian(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 1);
+	return __bm_count_bits(b, 1, KM_USER0);
 }
 
 void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line)
@@ -463,7 +444,7 @@ void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line)
 	spin_lock_irqsave(&b->bm_lock, flags);
 	bits = bm_count_bits(b);
 	if (bits != b->bm_set) {
-		ERR("bm_set was %lu, corrected to %lu. %s:%d\n",
+		dev_err(DEV, "bm_set was %lu, corrected to %lu. %s:%d\n",
 		    b->bm_set, bits, file, line);
 		b->bm_set = bits;
 	}
@@ -471,7 +452,7 @@ void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line)
 }
 
 /* offset and len in long words.*/
-STATIC void bm_memset(struct drbd_bitmap * b, size_t offset, int c, size_t len)
+STATIC void bm_memset(struct drbd_bitmap *b, size_t offset, int c, size_t len)
 {
 	unsigned long *p_addr, *bm;
 	size_t do_now, end;
@@ -517,17 +498,20 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 	unsigned long want, have, onpages; /* number of pages */
 	struct page **npages, **opages = NULL;
 	int err = 0, growing;
+	int opages_vmalloced;
 
 	ERR_IF(!b) return -ENOMEM;
 
 	drbd_bm_lock(mdev, "resize");
 
-	INFO("drbd_bm_resize called with capacity == %llu\n",
+	dev_info(DEV, "drbd_bm_resize called with capacity == %llu\n",
 			(unsigned long long)capacity);
 
 	if (capacity == b->bm_dev_capacity)
 		goto out;
 
+	opages_vmalloced = test_bit(BM_P_VMALLOCED, &b->bm_flags);
+
 	if (capacity == 0) {
 		spin_lock_irq(&b->bm_lock);
 		opages = b->bm_pages;
@@ -535,14 +519,13 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 		owords = b->bm_words;
 		b->bm_pages = NULL;
 		b->bm_number_of_pages =
-		b->bm_fo    =
 		b->bm_set   =
 		b->bm_bits  =
 		b->bm_words =
 		b->bm_dev_capacity = 0;
 		spin_unlock_irq(&b->bm_lock);
 		bm_free_pages(opages, onpages);
-		vfree(opages);
+		bm_vk_free(opages, opages_vmalloced);
 		goto out;
 	}
 	bits  = BM_SECT_TO_BIT(ALIGN(capacity, BM_SECT_PER_BIT));
@@ -554,9 +537,9 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 	*/
 	words = ALIGN(bits, 64) >> LN2_BPL;
 
-	if (inc_local(mdev)) {
-		D_ASSERT((u64)bits <= (((u64)mdev->bc->md.md_size_sect-MD_BM_OFFSET) << 12));
-		dec_local(mdev);
+	if (get_ldev(mdev)) {
+		D_ASSERT((u64)bits <= (((u64)mdev->ldev->md.md_size_sect-MD_BM_OFFSET) << 12));
+		put_ldev(mdev);
 	}
 
 	/* one extra long to catch off by one errors */
@@ -565,8 +548,12 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 	if (want == have) {
 		D_ASSERT(b->bm_pages != NULL);
 		npages = b->bm_pages;
-	} else
-		npages = bm_realloc_pages(b->bm_pages, have, want);
+	} else {
+		if (FAULT_ACTIVE(mdev, DRBD_FAULT_BM_ALLOC))
+			npages = NULL;
+		else
+			npages = bm_realloc_pages(b, want);
+	}
 
 	if (!npages) {
 		err = -ENOMEM;
@@ -609,11 +596,11 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 	if (!growing)
 		b->bm_set = bm_count_bits(b);
 
-	bm_end_info(mdev, __FUNCTION__);
+	bm_end_info(mdev, __func__);
 	spin_unlock_irq(&b->bm_lock);
 	if (opages != npages)
-		vfree(opages);
-	INFO("resync bitmap: bits=%lu words=%lu\n", bits, words);
+		bm_vk_free(opages, opages_vmalloced);
+	dev_info(DEV, "resync bitmap: bits=%lu words=%lu\n", bits, words);
 
  out:
 	drbd_bm_unlock(mdev);
@@ -634,6 +621,10 @@ unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
 	unsigned long s;
 	unsigned long flags;
 
+	/* if I don't have a disk, I don't know about out-of-sync status */
+	if (!get_ldev_if_state(mdev, D_NEGOTIATING))
+		return 0;
+
 	ERR_IF(!b) return 0;
 	ERR_IF(!b->bm_pages) return 0;
 
@@ -641,6 +632,8 @@ unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
 	s = b->bm_set;
 	spin_unlock_irqrestore(&b->bm_lock, flags);
 
+	put_ldev(mdev);
+
 	return s;
 }
 
@@ -730,7 +723,7 @@ void drbd_bm_get_lel(struct drbd_conf *mdev, size_t offset, size_t number,
 	if ((offset >= b->bm_words) ||
 	    (end    >  b->bm_words) ||
 	    (number <= 0))
-		ERR("offset=%lu number=%lu bm_words=%lu\n",
+		dev_err(DEV, "offset=%lu number=%lu bm_words=%lu\n",
 			(unsigned long)	offset,
 			(unsigned long)	number,
 			(unsigned long) b->bm_words);
@@ -794,9 +787,7 @@ static BIO_ENDIO_TYPE bm_async_io_complete BIO_ENDIO_ARGS(struct bio *bio, int e
 
 	if (error) {
 		/* doh. what now?
-		 * for now, set all bits, and flag MD_IO_ERROR
-		 */
-		/* FIXME kmap_atomic memset etc. pp. */
+		 * for now, set all bits, and flag MD_IO_ERROR */
 		__set_bit(BM_MD_IO_ERROR, &b->bm_flags);
 	}
 	if (atomic_dec_and_test(&b->bm_async_io))
@@ -813,18 +804,18 @@ STATIC void bm_page_io_async(struct drbd_conf *mdev, struct drbd_bitmap *b, int
 	struct bio *bio = bio_alloc(GFP_KERNEL, 1);
 	unsigned int len;
 	sector_t on_disk_sector =
-		mdev->bc->md.md_offset + mdev->bc->md.bm_offset;
+		mdev->ldev->md.md_offset + mdev->ldev->md.bm_offset;
 	on_disk_sector += ((sector_t)page_nr) << (PAGE_SHIFT-9);
 
 	/* this might happen with very small
 	 * flexible external meta data device */
 	len = min_t(unsigned int, PAGE_SIZE,
-		(drbd_md_last_sector(mdev->bc) - on_disk_sector + 1)<<9);
+		(drbd_md_last_sector(mdev->ldev) - on_disk_sector + 1)<<9);
 
 	D_DUMPLU(on_disk_sector);
 	D_DUMPI(len);
 
-	bio->bi_bdev = mdev->bc->md_bdev;
+	bio->bi_bdev = mdev->ldev->md_bdev;
 	bio->bi_sector = on_disk_sector;
 	bio_add_page(bio, b->bm_pages[page_nr], len, 0);
 	bio->bi_private = b;
@@ -848,24 +839,23 @@ void bm_cpu_to_lel(struct drbd_bitmap *b)
 	 * this may be optimized by using
 	 * cpu_to_lel(-1) == -1 and cpu_to_lel(0) == 0;
 	 * the following is still not optimal, but better than nothing */
+	unsigned int i;
+	unsigned long *p_addr, *bm;
 	if (b->bm_set == 0) {
 		/* no page at all; avoid swap if all is 0 */
 		i = b->bm_number_of_pages;
 	} else if (b->bm_set == b->bm_bits) {
 		/* only the last page */
-		i = b->bm_number_of_pages -1;
+		i = b->bm_number_of_pages - 1;
 	} else {
 		/* all pages */
 		i = 0;
 	}
 	for (; i < b->bm_number_of_pages; i++) {
-		unsigned long *bm;
-		/* if you'd want to use kmap_atomic, you'd have to disable irq! */
-		p_addr = kmap(b->bm_pages[i]);
-		for (bm = p_addr; bm < p_addr + PAGE_SIZE/sizeof(long); bm++) {
+		p_addr = kmap_atomic(b->bm_pages[i], KM_USER0);
+		for (bm = p_addr; bm < p_addr + PAGE_SIZE/sizeof(long); bm++)
 			*bm = cpu_to_lel(*bm);
-		}
-		kunmap(p_addr);
+		kunmap_atomic(p_addr, KM_USER0);
 	}
 }
 # endif
@@ -903,16 +893,11 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 	for (i = 0; i < num_pages; i++)
 		bm_page_io_async(mdev, b, i, rw);
 
-	drbd_blk_run_queue(bdev_get_queue(mdev->bc->md_bdev));
+	drbd_blk_run_queue(bdev_get_queue(mdev->ldev->md_bdev));
 	wait_event(b->bm_io_wait, atomic_read(&b->bm_async_io) == 0);
 
-	MTRACE(TraceTypeMDIO, TraceLvlSummary,
-	       INFO("%s of bitmap took %lu jiffies\n",
-		    rw == READ ? "reading" : "writing", jiffies - now);
-	       );
-
 	if (test_bit(BM_MD_IO_ERROR, &b->bm_flags)) {
-		ALERT("we had at least one MD IO ERROR during bitmap IO\n");
+		dev_alert(DEV, "we had at least one MD IO ERROR during bitmap IO\n");
 		drbd_chk_io_error(mdev, 1, TRUE);
 		drbd_io_error(mdev, TRUE);
 		err = -EIO;
@@ -927,21 +912,20 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 	} else /* rw == READ */ {
 		/* just read, if neccessary adjust endianness */
 		b->bm_set = bm_count_bits_swap_endian(b);
-		INFO("recounting of set bits took additional %lu jiffies\n",
+		dev_info(DEV, "recounting of set bits took additional %lu jiffies\n",
 		     jiffies - now);
 	}
 	now = b->bm_set;
 
-	INFO("%s (%lu bits) marked out-of-sync by on disk bit-map.\n",
-	     ppsize(ppb, now << (BM_BLOCK_SIZE_B-10)), now);
+	dev_info(DEV, "%s (%lu bits) marked out-of-sync by on disk bit-map.\n",
+	     ppsize(ppb, now << (BM_BLOCK_SHIFT-10)), now);
 
 	return err;
 }
 
 /**
- * drbd_bm_read: Read the whole bitmap from its on disk location.
- *
- * currently only called from "drbd_nl_disk_conf"
+ * drbd_bm_read() - Read the whole bitmap from its on disk location.
+ * @mdev:	DRBD device.
  */
 int drbd_bm_read(struct drbd_conf *mdev) __must_hold(local)
 {
@@ -949,9 +933,8 @@ int drbd_bm_read(struct drbd_conf *mdev) __must_hold(local)
 }
 
 /**
- * drbd_bm_write: Write the whole bitmap to its on disk location.
- *
- * called at various occasions.
+ * drbd_bm_write() - Write the whole bitmap to its on disk location.
+ * @mdev:	DRBD device.
  */
 int drbd_bm_write(struct drbd_conf *mdev) __must_hold(local)
 {
@@ -959,35 +942,37 @@ int drbd_bm_write(struct drbd_conf *mdev) __must_hold(local)
 }
 
 /**
- * drbd_bm_write_sect: Writes a 512 byte piece of the bitmap to its
- * on disk location. On disk bitmap is little endian.
- *
- * @enr: The _sector_ offset from the start of the bitmap.
+ * drbd_bm_write_sect: Writes a 512 (MD_SECTOR_SIZE) byte piece of the bitmap
+ * @mdev:	DRBD device.
+ * @enr:	Extent number in the resync lru (happens to be sector offset)
  *
+ * The BM_EXT_SIZE is on purpose exactle the amount of the bitmap covered
+ * by a single sector write. Therefore enr == sector offset from the
+ * start of the bitmap.
  */
 int drbd_bm_write_sect(struct drbd_conf *mdev, unsigned long enr) __must_hold(local)
 {
-	sector_t on_disk_sector = enr + mdev->bc->md.md_offset
-				      + mdev->bc->md.bm_offset;
+	sector_t on_disk_sector = enr + mdev->ldev->md.md_offset
+				      + mdev->ldev->md.bm_offset;
 	int bm_words, num_words, offset;
 	int err = 0;
 
-	down(&mdev->md_io_mutex);
+	mutex_lock(&mdev->md_io_mutex);
 	bm_words  = drbd_bm_words(mdev);
 	offset    = S2W(enr);	/* word offset into bitmap */
 	num_words = min(S2W(1), bm_words - offset);
 #if DUMP_MD >= 3
-	INFO("write_sect: sector=%lu offset=%u num_words=%u\n",
+	dev_info(DEV, "write_sect: sector=%lu offset=%u num_words=%u\n",
 			enr, offset, num_words);
 #endif
 	if (num_words < S2W(1))
-		memset(page_address(mdev->md_io_page), 0, MD_HARDSECT);
+		memset(page_address(mdev->md_io_page), 0, MD_SECTOR_SIZE);
 	drbd_bm_get_lel(mdev, offset, num_words,
 			page_address(mdev->md_io_page));
-	if (!drbd_md_sync_page_io(mdev, mdev->bc, on_disk_sector, WRITE)) {
+	if (!drbd_md_sync_page_io(mdev, mdev->ldev, on_disk_sector, WRITE)) {
 		int i;
 		err = -EIO;
-		ERR("IO ERROR writing bitmap sector %lu "
+		dev_err(DEV, "IO ERROR writing bitmap sector %lu "
 		    "(meta-disk sector %llus)\n",
 		    enr, (unsigned long long)on_disk_sector);
 		drbd_chk_io_error(mdev, 1, TRUE);
@@ -996,24 +981,10 @@ int drbd_bm_write_sect(struct drbd_conf *mdev, unsigned long enr) __must_hold(lo
 			drbd_bm_ALe_set_all(mdev, enr*AL_EXT_PER_BM_SECT+i);
 	}
 	mdev->bm_writ_cnt++;
-	up(&mdev->md_io_mutex);
+	mutex_unlock(&mdev->md_io_mutex);
 	return err;
 }
 
-void drbd_bm_reset_find(struct drbd_conf *mdev)
-{
-	struct drbd_bitmap *b = mdev->bitmap;
-
-	ERR_IF(!b) return;
-
-	spin_lock_irq(&b->bm_lock);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
-	b->bm_fo = 0;
-	spin_unlock_irq(&b->bm_lock);
-
-}
-
 /* NOTE
  * find_first_bit returns int, we return unsigned long.
  * should not make much difference anyways, but ...
@@ -1021,93 +992,114 @@ void drbd_bm_reset_find(struct drbd_conf *mdev)
  * this returns a bit number, NOT a sector!
  */
 #define BPP_MASK ((1UL << (PAGE_SHIFT+3)) - 1)
-unsigned long drbd_bm_find_next(struct drbd_conf *mdev)
+static unsigned long __bm_find_next(struct drbd_conf *mdev, unsigned long bm_fo,
+	const int find_zero_bit, const enum km_type km)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long i = -1UL;
 	unsigned long *p_addr;
 	unsigned long bit_offset; /* bit offset of the mapped page. */
 
+	if (bm_fo > b->bm_bits) {
+		dev_err(DEV, "bm_fo=%lu bm_bits=%lu\n", bm_fo, b->bm_bits);
+	} else {
+		while (bm_fo < b->bm_bits) {
+			unsigned long offset;
+			bit_offset = bm_fo & ~BPP_MASK; /* bit offset of the page */
+			offset = bit_offset >> LN2_BPL;    /* word offset of the page */
+			p_addr = __bm_map_paddr(b, offset, km);
+
+			if (find_zero_bit)
+				i = find_next_zero_bit(p_addr, PAGE_SIZE*8, bm_fo & BPP_MASK);
+			else
+				i = find_next_bit(p_addr, PAGE_SIZE*8, bm_fo & BPP_MASK);
+
+			__bm_unmap(p_addr, km);
+			if (i < PAGE_SIZE*8) {
+				i = bit_offset + i;
+				if (i >= b->bm_bits)
+					break;
+				goto found;
+			}
+			bm_fo = bit_offset + PAGE_SIZE*8;
+		}
+		i = -1UL;
+	}
+ found:
+	return i;
+}
+
+static unsigned long bm_find_next(struct drbd_conf *mdev,
+	unsigned long bm_fo, const int find_zero_bit)
+{
+	struct drbd_bitmap *b = mdev->bitmap;
+	unsigned long i = -1UL;
+
 	ERR_IF(!b) return i;
 	ERR_IF(!b->bm_pages) return i;
 
 	spin_lock_irq(&b->bm_lock);
 	if (bm_is_locked(b))
 		bm_print_lock_info(mdev);
-	if (b->bm_fo > b->bm_bits) {
-		ERR("bm_fo=%lu bm_bits=%lu\n", b->bm_fo, b->bm_bits);
-	} else {
-		while (b->bm_fo < b->bm_bits) {
-			unsigned long offset;
-			bit_offset = b->bm_fo & ~BPP_MASK; /* bit offset of the page */
-			offset = bit_offset >> LN2_BPL;    /* word offset of the page */
-			p_addr = bm_map_paddr(b, offset);
-			i = find_next_bit(p_addr, PAGE_SIZE*8, b->bm_fo & BPP_MASK);
-			bm_unmap(p_addr);
-			if (i < PAGE_SIZE*8) {
-				i = bit_offset + i;
-				if (i >= b->bm_bits)
-					break;
-				b->bm_fo = i+1;
-				goto found;
-			}
-			b->bm_fo = bit_offset + PAGE_SIZE*8;
-		}
-		i = -1UL;
-		/* leave b->bm_fo unchanged. */
-	}
- found:
+
+	i = __bm_find_next(mdev, bm_fo, find_zero_bit, KM_IRQ1);
+
 	spin_unlock_irq(&b->bm_lock);
 	return i;
 }
 
-void drbd_bm_set_find(struct drbd_conf *mdev, unsigned long i)
+unsigned long drbd_bm_find_next(struct drbd_conf *mdev, unsigned long bm_fo)
 {
-	struct drbd_bitmap *b = mdev->bitmap;
-
-	spin_lock_irq(&b->bm_lock);
-
-	b->bm_fo = min_t(unsigned long, i, b->bm_bits);
-
-	spin_unlock_irq(&b->bm_lock);
+	return bm_find_next(mdev, bm_fo, 0);
 }
 
-int drbd_bm_rs_done(struct drbd_conf *mdev)
+#if 0
+/* not yet needed for anything. */
+unsigned long drbd_bm_find_next_zero(struct drbd_conf *mdev, unsigned long bm_fo)
 {
-	D_ASSERT(mdev->bitmap);
-	return mdev->bitmap->bm_fo >= mdev->bitmap->bm_bits;
+	return bm_find_next(mdev, bm_fo, 1);
+}
+#endif
+
+/* does not spin_lock_irqsave.
+ * you must take drbd_bm_lock() first */
+unsigned long _drbd_bm_find_next(struct drbd_conf *mdev, unsigned long bm_fo)
+{
+	/* WARN_ON(!bm_is_locked(mdev)); */
+	return __bm_find_next(mdev, bm_fo, 0, KM_USER1);
+}
+
+unsigned long _drbd_bm_find_next_zero(struct drbd_conf *mdev, unsigned long bm_fo)
+{
+	/* WARN_ON(!bm_is_locked(mdev)); */
+	return __bm_find_next(mdev, bm_fo, 1, KM_USER1);
 }
 
 /* returns number of bits actually changed.
  * for val != 0, we change 0 -> 1, return code positiv
  * for val == 0, we change 1 -> 0, return code negative
- * wants bitnr, not sector */
-static int bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
-	const unsigned long e, int val)
+ * wants bitnr, not sector.
+ * Must hold bitmap lock already. */
+
+int __bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
+	const unsigned long e, int val, const enum km_type km)
 {
-	unsigned long flags;
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long *p_addr = NULL;
 	unsigned long bitnr;
 	unsigned long last_page_nr = -1UL;
 	int c = 0;
 
-	ERR_IF(!b) return 1;
-	ERR_IF(!b->bm_pages) return 0;
-
-	spin_lock_irqsave(&b->bm_lock, flags);
-	if (bm_is_locked(b))
-		bm_print_lock_info(mdev);
 	for (bitnr = s; bitnr <= e; bitnr++) {
 		ERR_IF (bitnr >= b->bm_bits) {
-			ERR("bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
+			dev_err(DEV, "bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
 		} else {
 			unsigned long offset = bitnr>>LN2_BPL;
 			unsigned long page_nr = offset >> (PAGE_SHIFT - LN2_BPL + 3);
 			if (page_nr != last_page_nr) {
 				if (p_addr)
-					bm_unmap(p_addr);
-				p_addr = bm_map_paddr(b, offset);
+					__bm_unmap(p_addr, km);
+				p_addr = __bm_map_paddr(b, offset, km);
 				last_page_nr = page_nr;
 			}
 			if (val)
@@ -1117,8 +1109,31 @@ static int bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
 		}
 	}
 	if (p_addr)
-		bm_unmap(p_addr);
+		__bm_unmap(p_addr, km);
 	b->bm_set += c;
+	return c;
+}
+
+/* returns number of bits actually changed.
+ * for val != 0, we change 0 -> 1, return code positiv
+ * for val == 0, we change 1 -> 0, return code negative
+ * wants bitnr, not sector */
+int bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
+	const unsigned long e, int val)
+{
+	unsigned long flags;
+	struct drbd_bitmap *b = mdev->bitmap;
+	int c = 0;
+
+	ERR_IF(!b) return 1;
+	ERR_IF(!b->bm_pages) return 0;
+
+	spin_lock_irqsave(&b->bm_lock, flags);
+	if (bm_is_locked(b))
+		bm_print_lock_info(mdev);
+
+	c = __bm_change_bits_to(mdev, s, e, val, KM_IRQ1);
+
 	spin_unlock_irqrestore(&b->bm_lock, flags);
 	return c;
 }
@@ -1135,6 +1150,14 @@ int drbd_bm_clear_bits(struct drbd_conf *mdev, const unsigned long s, const unsi
 	return -bm_change_bits_to(mdev, s, e, 0);
 }
 
+/* the same thing, but without taking the spin_lock_irqsave.
+ * you must first drbd_bm_lock(). */
+int _drbd_bm_set_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e)
+{
+       /* WARN_ON(!bm_is_locked(b)); */
+       return __bm_change_bits_to(mdev, s, e, 1, KM_USER0);
+}
+
 /* returns bit state
  * wants bitnr, NOT sector.
  * inherently racy... area needs to be locked by means of {al,rs}_lru
@@ -1163,7 +1186,7 @@ int drbd_bm_test_bit(struct drbd_conf *mdev, const unsigned long bitnr)
 	} else if (bitnr == b->bm_bits) {
 		i = -1;
 	} else { /* (bitnr > b->bm_bits) */
-		ERR("bitnr=%lu > bm_bits=%lu\n", bitnr, b->bm_bits);
+		dev_err(DEV, "bitnr=%lu > bm_bits=%lu\n", bitnr, b->bm_bits);
 		i = 0;
 	}
 
@@ -1198,7 +1221,7 @@ int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsi
 			p_addr = bm_map_paddr(b, w);
 		}
 		ERR_IF (bitnr >= b->bm_bits) {
-			ERR("bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
+			dev_err(DEV, "bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
 		} else {
 			c += (0 != test_bit(bitnr - (page_nr << (PAGE_SHIFT+3)), p_addr));
 		}
@@ -1252,11 +1275,11 @@ int drbd_bm_e_weight(struct drbd_conf *mdev, unsigned long enr)
 		}
 		bm_unmap(p_addr);
 	} else {
-		ERR("start offset (%d) too large in drbd_bm_e_weight\n", s);
+		dev_err(DEV, "start offset (%d) too large in drbd_bm_e_weight\n", s);
 	}
 	spin_unlock_irqrestore(&b->bm_lock, flags);
 #if DUMP_MD >= 3
-	INFO("enr=%lu weight=%d e=%d s=%d\n", enr, count, e, s);
+	dev_info(DEV, "enr=%lu weight=%d e=%d s=%d\n", enr, count, e, s);
 #endif
 	return count;
 }
@@ -1298,7 +1321,7 @@ unsigned long drbd_bm_ALe_set_all(struct drbd_conf *mdev, unsigned long al_enr)
 		if (e == b->bm_words)
 			b->bm_set -= bm_clear_surplus(b);
 	} else {
-		ERR("start offset (%d) too large in drbd_bm_ALe_set_all\n", s);
+		dev_err(DEV, "start offset (%d) too large in drbd_bm_ALe_set_all\n", s);
 	}
 	weight = b->bm_set - weight;
 	spin_unlock_irq(&b->bm_lock);
