@@ -1,7 +1,5 @@
 /*
--*- linux-c -*-
    drbd_req.c
-   Kernel module for 2.6.x Kernels
 
    This file is part of DRBD by Philipp Reisner and Lars Ellenberg.
 
@@ -31,82 +29,9 @@
 #include <linux/slab.h>
 #include <linux/drbd.h>
 #include "drbd_int.h"
+#include "drbd_tracing.h"
 #include "drbd_req.h"
 
-/* outside of the ifdef
- * because of the _print_rq_state(,FIXME) in barrier_acked */
-STATIC void _print_rq_state(struct drbd_request *req, const char *txt)
-{
-	const unsigned long s = req->rq_state;
-	struct drbd_conf *mdev = req->mdev;
-	const int rw = (req->master_bio == NULL ||
-			bio_data_dir(req->master_bio) == WRITE) ?
-		'W' : 'R';
-
-	INFO("%s %p %c L%c%c%cN%c%c%c%c%c %u (%llus +%u) %s\n",
-	     txt, req, rw,
-	     s & RQ_LOCAL_PENDING ? 'p' : '-',
-	     s & RQ_LOCAL_COMPLETED ? 'c' : '-',
-	     s & RQ_LOCAL_OK ? 'o' : '-',
-	     s & RQ_NET_PENDING ? 'p' : '-',
-	     s & RQ_NET_QUEUED ? 'q' : '-',
-	     s & RQ_NET_SENT ? 's' : '-',
-	     s & RQ_NET_DONE ? 'd' : '-',
-	     s & RQ_NET_OK ? 'o' : '-',
-	     req->epoch,
-	     (unsigned long long)req->sector,
-	     req->size,
-	     conns_to_name(mdev->state.conn));
-}
-
-/* #define VERBOSE_REQUEST_CODE */
-#if defined(VERBOSE_REQUEST_CODE) || defined(ENABLE_DYNAMIC_TRACE)
-STATIC void _print_req_mod(struct drbd_request *req, enum drbd_req_event what)
-{
-	struct drbd_conf *mdev = req->mdev;
-	const int rw = (req->master_bio == NULL ||
-			bio_data_dir(req->master_bio) == WRITE) ?
-		'W' : 'R';
-
-	static const char *rq_event_names[] = {
-		[created] = "created",
-		[to_be_send] = "to_be_send",
-		[to_be_submitted] = "to_be_submitted",
-		[queue_for_net_write] = "queue_for_net_write",
-		[queue_for_net_read] = "queue_for_net_read",
-		[send_canceled] = "send_canceled",
-		[send_failed] = "send_failed",
-		[handed_over_to_network] = "handed_over_to_network",
-		[connection_lost_while_pending] =
-					"connection_lost_while_pending",
-		[recv_acked_by_peer] = "recv_acked_by_peer",
-		[write_acked_by_peer] = "write_acked_by_peer",
-		[neg_acked] = "neg_acked",
-		[conflict_discarded_by_peer] = "conflict_discarded_by_peer",
-		[barrier_acked] = "barrier_acked",
-		[data_received] = "data_received",
-		[read_completed_with_error] = "read_completed_with_error",
-		[write_completed_with_error] = "write_completed_with_error",
-		[completed_ok] = "completed_ok",
-	};
-
-	INFO("_req_mod(%p %c ,%s)\n", req, rw, rq_event_names[what]);
-}
-
-# ifdef ENABLE_DYNAMIC_TRACE
-#  define print_rq_state(R, T) \
-	MTRACE(TraceTypeRq, TraceLvlMetrics, _print_rq_state(R, T);)
-#  define print_req_mod(T, W)  \
-	MTRACE(TraceTypeRq, TraceLvlMetrics, _print_req_mod(T, W);)
-# else
-#  define print_rq_state(R, T) _print_rq_state(R, T)
-#  define print_req_mod(T, W)  _print_req_mod(T, W)
-# endif
-
-#else
-#define print_rq_state(R, T)
-#define print_req_mod(T, W)
-#endif
 
 /* We only support diskstats for 2.6.16 and up.
  * see also commit commit a362357b6cd62643d4dda3b152639303d78473da
@@ -119,27 +44,47 @@ STATIC void _print_req_mod(struct drbd_request *req, enum drbd_req_event what)
 #else
 
 /* Update disk stats at start of I/O request */
-static inline void _drbd_start_io_acct(struct drbd_conf *mdev, struct drbd_request *req, struct bio *bio)
+static void _drbd_start_io_acct(struct drbd_conf *mdev, struct drbd_request *req, struct bio *bio)
 {
 	const int rw = bio_data_dir(bio);
+#ifndef __disk_stat_inc
+	int cpu;
+#endif
 
-	MUST_HOLD(&mdev->req_lock)
+#ifdef __disk_stat_inc
 	__disk_stat_inc(mdev->vdisk, ios[rw]);
 	__disk_stat_add(mdev->vdisk, sectors[rw], bio_sectors(bio));
 	disk_round_stats(mdev->vdisk);
 	mdev->vdisk->in_flight++;
+#else
+	cpu = part_stat_lock();
+	part_stat_inc(cpu, &mdev->vdisk->part0, ios[rw]);
+	part_stat_add(cpu, &mdev->vdisk->part0, sectors[rw], bio_sectors(bio));
+	part_stat_unlock();
+	mdev->vdisk->part0.in_flight++;
+#endif
 }
 
 /* Update disk stats when completing request upwards */
-static inline void _drbd_end_io_acct(struct drbd_conf *mdev, struct drbd_request *req)
+static void _drbd_end_io_acct(struct drbd_conf *mdev, struct drbd_request *req)
 {
 	int rw = bio_data_dir(req->master_bio);
 	unsigned long duration = jiffies - req->start_time;
+#ifndef __disk_stat_inc
+	int cpu;
+#endif
 
-	MUST_HOLD(&mdev->req_lock)
+#ifdef __disk_stat_add
 	__disk_stat_add(mdev->vdisk, ticks[rw], duration);
 	disk_round_stats(mdev->vdisk);
 	mdev->vdisk->in_flight--;
+#else
+	cpu = part_stat_lock();
+	part_stat_add(cpu, &mdev->vdisk->part0, ticks[rw], duration);
+	part_round_stats(cpu, &mdev->vdisk->part0);
+	part_stat_unlock();
+	mdev->vdisk->part0.in_flight--;
+#endif
 }
 
 #endif
@@ -179,11 +124,11 @@ static void _req_is_done(struct drbd_conf *mdev, struct drbd_request *req, const
 		 * we would forget to resync the corresponding extent.
 		 */
 		if (s & RQ_LOCAL_MASK) {
-			if (inc_local_if_state(mdev, Failed)) {
+			if (get_ldev_if_state(mdev, D_FAILED)) {
 				drbd_al_complete_io(mdev, req->sector);
-				dec_local(mdev);
-			} else {
-				WARN("Should have called drbd_al_complete_io(, %llu), "
+				put_ldev(mdev);
+			} else if (DRBD_ratelimit(5*HZ, 3)) {
+				dev_warn(DEV, "Should have called drbd_al_complete_io(, %llu), "
 				     "but my Disk seems to have failed :(\n",
 				     (unsigned long long) req->sector);
 			}
@@ -202,7 +147,7 @@ static void _req_is_done(struct drbd_conf *mdev, struct drbd_request *req, const
 	 * until the next barrier ack? */
 
 	if (rw == WRITE &&
-	    (( s & RQ_LOCAL_MASK) && !(s & RQ_LOCAL_OK))) {
+	    ((s & RQ_LOCAL_MASK) && !(s & RQ_LOCAL_OK))) {
 		if (!(req->w.list.next == LIST_POISON1 ||
 		      list_empty(&req->w.list))) {
 			/* DEBUG ASSERT only; if this triggers, we
@@ -220,7 +165,7 @@ static void _req_is_done(struct drbd_conf *mdev, struct drbd_request *req, const
 
 static void queue_barrier(struct drbd_conf *mdev)
 {
-	struct drbd_barrier *b;
+	struct drbd_tl_epoch *b;
 
 	/* We are within the req_lock. Once we queued the barrier for sending,
 	 * we set the CREATE_BARRIER bit. It is cleared as soon as a new
@@ -230,7 +175,7 @@ static void queue_barrier(struct drbd_conf *mdev)
 	if (test_bit(CREATE_BARRIER, &mdev->flags))
 		return;
 
-	b = mdev->newest_barrier;
+	b = mdev->newest_tle;
 	b->w.cb = w_send_barrier;
 	/* inc_ap_pending done here, so we won't
 	 * get imbalanced on connection loss.
@@ -246,14 +191,14 @@ static void _about_to_complete_local_write(struct drbd_conf *mdev,
 {
 	const unsigned long s = req->rq_state;
 	struct drbd_request *i;
-	struct Tl_epoch_entry *e;
+	struct drbd_epoch_entry *e;
 	struct hlist_node *n;
 	struct hlist_head *slot;
 
 	/* before we can signal completion to the upper layers,
 	 * we may need to close the current epoch */
-	if (mdev->state.conn >= Connected &&
-	    req->epoch == mdev->newest_barrier->br_number)
+	if (mdev->state.conn >= C_CONNECTED &&
+	    req->epoch == mdev->newest_tle->br_number)
 		queue_barrier(mdev);
 
 	/* we need to do the conflict detection stuff,
@@ -270,7 +215,7 @@ static void _about_to_complete_local_write(struct drbd_conf *mdev,
 		slot = tl_hash_slot(mdev, sector);
 		hlist_for_each_entry(i, n, slot, colision) {
 			if (OVERLAPS) {
-				ALERT("LOGIC BUG: completed: %p %llus +%u; "
+				dev_alert(DEV, "LOGIC BUG: completed: %p %llus +%u; "
 				      "other: %p %llus +%u\n",
 				      req, (unsigned long long)sector, size,
 				      i, (unsigned long long)i->sector, i->size);
@@ -282,7 +227,7 @@ static void _about_to_complete_local_write(struct drbd_conf *mdev,
 		 *
 		 * currently, there can be only _one_ such ee
 		 * (well, or some more, which would be pending
-		 * DiscardAck not yet sent by the asender...),
+		 * P_DISCARD_ACK not yet sent by the asender...),
 		 * since we block the receiver thread upon the
 		 * first conflict detection, which will wait on
 		 * misc_wait.  maybe we want to assert that?
@@ -305,7 +250,7 @@ static void _about_to_complete_local_write(struct drbd_conf *mdev,
 static void _complete_master_bio(struct drbd_conf *mdev,
 	struct drbd_request *req, int error)
 {
-	dump_bio(mdev, req->master_bio, 1);
+	trace_drbd_bio(mdev, "Rq", req->master_bio, 1, req);
 	bio_endio(req->master_bio, error);
 	req->master_bio = NULL;
 	dec_ap_bio(mdev);
@@ -317,8 +262,7 @@ void _req_may_be_done(struct drbd_request *req, int error)
 	struct drbd_conf *mdev = req->mdev;
 	int rw;
 
-	print_rq_state(req, "_req_may_be_done");
-	MUST_HOLD(&mdev->req_lock)
+	trace_drbd_req(req, nothing, "_req_may_be_done");
 
 	/* we must not complete the master bio, while it is
 	 *	still being processed by _drbd_send_zc_bio (drbd_send_dblock)
@@ -338,8 +282,8 @@ void _req_may_be_done(struct drbd_request *req, int error)
 
 	if (req->master_bio) {
 		/* this is data_received (remote read)
-		 * or protocol C WriteAck
-		 * or protocol B RecvAck
+		 * or protocol C P_WRITE_ACK
+		 * or protocol B P_RECV_ACK
 		 * or protocol A "handed_over_to_network" (SendAck)
 		 * or canceled or failed,
 		 * or killed from the transfer log due to connection loss.
@@ -369,20 +313,11 @@ void _req_may_be_done(struct drbd_request *req, int error)
 		if (rw == WRITE)
 			_about_to_complete_local_write(mdev, req);
 
-		/* FIXME not yet implemented...
-		 * in case we got "suspended" (on_disconnect: freeze io)
-		 * we may not yet complete the request...
-		 * though, this is probably best handled elsewhere by not
-		 * walking the transfer log until "unfreeze", so we won't end
-		 * up here anyways during the freeze ...
-		 * then again, if it is a READ, it is not in the TL at all.
-		 * is it still leagal to complete a READ during freeze? */
-
 		/* Update disk stats */
 		_drbd_end_io_acct(mdev, req);
 
-		_complete_master_bio(mdev,req,
-				     ok ? 0 : ( error ? error : -EIO ) );
+		_complete_master_bio(mdev, req,
+				     ok ? 0 : (error ? error : -EIO));
 	} else {
 		/* only WRITE requests can end up here without a master_bio */
 		rw = WRITE;
@@ -390,8 +325,8 @@ void _req_may_be_done(struct drbd_request *req, int error)
 
 	if ((s & RQ_NET_MASK) == 0 || (s & RQ_NET_DONE)) {
 		/* this is disconnected (local only) operation,
-		 * or protocol C WriteAck,
-		 * or protocol A or B BarrierAck,
+		 * or protocol C P_WRITE_ACK,
+		 * or protocol A or B P_BARRIER_ACK,
 		 * or killed from the transfer log due to connection loss. */
 		_req_is_done(mdev, req, rw);
 	}
@@ -428,16 +363,13 @@ STATIC int _req_conflicts(struct drbd_request *req)
 	const sector_t sector = req->sector;
 	const int size = req->size;
 	struct drbd_request *i;
-	struct Tl_epoch_entry *e;
+	struct drbd_epoch_entry *e;
 	struct hlist_node *n;
 	struct hlist_head *slot;
 
-	MUST_HOLD(&mdev->req_lock);
 	D_ASSERT(hlist_unhashed(&req->colision));
 
-	/* FIXME should this inc_net/dec_net
-	 * rather be done in drbd_make_request_common? */
-	if (!inc_net(mdev))
+	if (!get_net_conf(mdev))
 		return 0;
 
 	/* BUG_ON */
@@ -449,7 +381,7 @@ STATIC int _req_conflicts(struct drbd_request *req)
 	slot = tl_hash_slot(mdev, sector);
 	hlist_for_each_entry(i, n, slot, colision) {
 		if (OVERLAPS) {
-			ALERT("%s[%u] Concurrent local write detected! "
+			dev_alert(DEV, "%s[%u] Concurrent local write detected! "
 			      "[DISCARD L] new: %llus +%u; "
 			      "pending: %llus +%u\n",
 			      current->comm, current->pid,
@@ -467,7 +399,7 @@ STATIC int _req_conflicts(struct drbd_request *req)
 		slot = ee_hash_slot(mdev, sector);
 		hlist_for_each_entry(e, n, slot, colision) {
 			if (OVERLAPS) {
-				ALERT("%s[%u] Concurrent remote write detected!"
+				dev_alert(DEV, "%s[%u] Concurrent remote write detected!"
 				      " [DISCARD L] new: %llus +%u; "
 				      "pending: %llus +%u\n",
 				      current->comm, current->pid,
@@ -482,11 +414,11 @@ STATIC int _req_conflicts(struct drbd_request *req)
 out_no_conflict:
 	/* this is like it should be, and what we expected.
 	 * our users do behave after all... */
-	dec_net(mdev);
+	put_net_conf(mdev);
 	return 0;
 
 out_conflict:
-	dec_net(mdev);
+	put_net_conf(mdev);
 	return 1;
 }
 
@@ -501,23 +433,19 @@ out_conflict:
  *  happen "atomically" within the req_lock,
  *  and it enforces that we have to think in a very structured manner
  *  about the "events" that may happen to a request during its life time ...
- *
- * Though I think it is likely that we break this again into many
- * static inline void _req_mod_ ## what (req) ...
  */
 void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 {
 	struct drbd_conf *mdev = req->mdev;
-	MUST_HOLD(&mdev->req_lock);
 
 	if (error && (bio_rw(req->master_bio) != READA))
-		ERR("got an _req_mod() errno of %d\n", error);
+		dev_err(DEV, "got an _req_mod() errno of %d\n", error);
 
-	print_req_mod(req, what);
+	trace_drbd_req(req, what, NULL);
 
 	switch (what) {
 	default:
-		ERR("LOGIC BUG in %s:%u\n", __FILE__ , __LINE__ );
+		dev_err(DEV, "LOGIC BUG in %s:%u\n", __FILE__ , __LINE__);
 		return;
 
 	/* does not happen...
@@ -528,7 +456,7 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 
 	case to_be_send: /* via network */
 		/* reached via drbd_make_request_common
-		 * and from FIXME w_read_retry_remote */
+		 * and from w_read_retry_remote */
 		D_ASSERT(!(req->rq_state & RQ_NET_MASK));
 		req->rq_state |= RQ_NET_PENDING;
 		inc_ap_pending(mdev);
@@ -540,9 +468,6 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		req->rq_state |= RQ_LOCAL_PENDING;
 		break;
 
-	/* FIXME these *_completed_* are basically the same.
-	 * can probably be merged with some if (what == xy) */
-
 	case completed_ok:
 		if (bio_data_dir(req->private_bio) == WRITE)
 			mdev->writ_cnt += req->size>>9;
@@ -556,7 +481,7 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		req->rq_state &= ~RQ_LOCAL_PENDING;
 
 		_req_may_be_done(req, error);
-		dec_local(mdev);
+		put_ldev(mdev);
 		break;
 
 	case write_completed_with_error:
@@ -565,13 +490,12 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 
 		bio_put(req->private_bio);
 		req->private_bio = NULL;
-		ALERT("Local WRITE failed sec=%llus size=%u\n",
+		dev_alert(DEV, "Local WRITE failed sec=%llus size=%u\n",
 		      (unsigned long long)req->sector, req->size);
-		/* and now: check how to handle local io error.
-		 * FIXME see comment below in read_completed_with_error */
+		/* and now: check how to handle local io error. */
 		__drbd_chk_io_error(mdev, FALSE);
 		_req_may_be_done(req, error);
-		dec_local(mdev);
+		put_ldev(mdev);
 		break;
 
 	case read_completed_with_error:
@@ -586,28 +510,19 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		if (bio_rw(req->master_bio) == READA) {
 			/* it is legal to fail READA */
 			_req_may_be_done(req, error);
-			dec_local(mdev);
+			put_ldev(mdev);
 			break;
 		}
 		/* else */
-		ALERT("Local READ failed sec=%llus size=%u\n",
+		dev_alert(DEV, "Local READ failed sec=%llus size=%u\n",
 		      (unsigned long long)req->sector, req->size);
-		/* _req_mod(req,to_be_send); oops, recursion in static inline */
+		/* _req_mod(req,to_be_send); oops, recursion... */
 		D_ASSERT(!(req->rq_state & RQ_NET_MASK));
 		req->rq_state |= RQ_NET_PENDING;
 		inc_ap_pending(mdev);
 
-		/* and now: check how to handle local io error.
-		 *
-		 * FIXME we should not handle WRITE and READ io errors
-		 * the same. When we retry the READ, and then write
-		 * the answer, that might suceed because modern drives
-		 * would relocate the sectors. We'd need to keep our
-		 * private bio then, and round the offset and size so
-		 * we get back enough data to be able to clear the bits again.
-		 */
 		__drbd_chk_io_error(mdev, FALSE);
-		dec_local(mdev);
+		put_ldev(mdev);
 		/* NOTE: if we have no connection,
 		 * or know the peer has no good data either,
 		 * then we don't actually need to "queue_for_net_read",
@@ -664,12 +579,12 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		 * just after it grabs the req_lock */
 		D_ASSERT(test_bit(CREATE_BARRIER, &mdev->flags) == 0);
 
-		req->epoch = mdev->newest_barrier->br_number;
+		req->epoch = mdev->newest_tle->br_number;
 		list_add_tail(&req->tl_requests,
-				&mdev->newest_barrier->requests);
+				&mdev->newest_tle->requests);
 
 		/* increment size of current epoch */
-		mdev->newest_barrier->n_req++;
+		mdev->newest_tle->n_req++;
 
 		/* queue work item to send data */
 		D_ASSERT(req->rq_state & RQ_NET_PENDING);
@@ -678,20 +593,13 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		drbd_queue_work(&mdev->data.work, &req->w);
 
 		/* close the epoch, in case it outgrew the limit */
-		if (mdev->newest_barrier->n_req >= mdev->net_conf->max_epoch_size)
+		if (mdev->newest_tle->n_req >= mdev->net_conf->max_epoch_size)
 			queue_barrier(mdev);
 
 		break;
 
-	/* FIXME
-	 * to implement freeze-io,
-	 * we may not finish the request just yet.
-	 */
 	case send_canceled:
-		/* may be a write, may be a remote read */
-		if (req->rq_state & RQ_NET_PENDING) dec_ap_pending(mdev);
-		req->rq_state &= ~(RQ_NET_OK|RQ_NET_PENDING);
-		/* fall through */
+		/* treat it the same */
 	case send_failed:
 		/* real cleanup will be done from tl_clear.  just update flags
 		 * so it is no longer marked as on the worker queue */
@@ -703,16 +611,10 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 
 	case handed_over_to_network:
 		/* assert something? */
-		if ( bio_data_dir(req->master_bio) == WRITE &&
-		     mdev->net_conf->wire_protocol == DRBD_PROT_A ) {
+		if (bio_data_dir(req->master_bio) == WRITE &&
+		    mdev->net_conf->wire_protocol == DRBD_PROT_A) {
 			/* this is what is dangerous about protocol A:
-			 * pretend it was sucessfully written on the peer.
-			 * FIXME in case we get a local io-error in
-			 * protocol != C, we might want to defer comletion
-			 * until we get the barrier ack, and send a NegAck
-			 * in case the other node had an io-error, too...
-			 * That way we would at least not report "success"
-			 * if it was not written at all. */
+			 * pretend it was sucessfully written on the peer. */
 			if (req->rq_state & RQ_NET_PENDING) {
 				dec_ap_pending(mdev);
 				req->rq_state &= ~RQ_NET_PENDING;
@@ -740,8 +642,7 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		req->rq_state &= ~(RQ_NET_OK|RQ_NET_PENDING);
 		req->rq_state |= RQ_NET_DONE;
 		/* if it is still queued, we may not complete it here.
-		 * it will be canceled soon.
-		 * FIXME we should change the code so this can not happen. */
+		 * it will be canceled soon. */
 		if (!(req->rq_state & RQ_NET_QUEUED))
 			_req_may_be_done(req, error);
 		break;
@@ -763,7 +664,7 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		 * A barrier request is expected to have forced all prior
 		 * requests onto stable storage, so completion of a barrier
 		 * request could set NET_DONE right here, and not wait for the
-		 * BarrierAck, but that is an unecessary optimisation. */
+		 * P_BARRIER_ACK, but that is an unecessary optimisation. */
 
 		/* this makes it effectively the same as for: */
 	case recv_acked_by_peer:
@@ -782,7 +683,7 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 		if (req->rq_state & RQ_NET_PENDING)
 			dec_ap_pending(mdev);
 		req->rq_state &= ~(RQ_NET_OK|RQ_NET_PENDING);
-		/* FIXME THINK! is it DONE now, or is it not? */
+
 		req->rq_state |= RQ_NET_DONE;
 		_req_may_be_done(req, error);
 		/* else: done by handed_over_to_network */
@@ -793,8 +694,9 @@ void _req_mod(struct drbd_request *req, enum drbd_req_event what, int error)
 			/* barrier came in before all requests have been acked.
 			 * this is bad, because if the connection is lost now,
 			 * we won't be able to clean them up... */
-			_print_rq_state(req,
-				"FIXME (barrier_acked but pending)");
+			dev_err(DEV, "FIXME (barrier_acked but pending)\n");
+			trace_drbd_req(req, nothing, "FIXME (barrier_acked but pending)");
+			list_move(&req->tl_requests, &mdev->out_of_sequence_requests);
 		}
 		D_ASSERT(req->rq_state & RQ_NET_SENT);
 		req->rq_state |= RQ_NET_DONE;
@@ -823,15 +725,15 @@ STATIC int drbd_may_do_local_read(struct drbd_conf *mdev, sector_t sector, int s
 	unsigned long sbnr, ebnr;
 	sector_t esector, nr_sectors;
 
-	if (mdev->state.disk == UpToDate)
+	if (mdev->state.disk == D_UP_TO_DATE)
 		return 1;
-	if (mdev->state.disk >= Outdated)
+	if (mdev->state.disk >= D_OUTDATED)
 		return 0;
-	if (mdev->state.disk <  Inconsistent)
+	if (mdev->state.disk <  D_INCONSISTENT)
 		return 0;
-	/* state.disk == Inconsistent   We will have a look at the BitMap */
+	/* state.disk == D_INCONSISTENT   We will have a look at the BitMap */
 	nr_sectors = drbd_get_capacity(mdev->this_bdev);
-	esector = sector + (size>>9) -1;
+	esector = sector + (size >> 9) - 1;
 
 	D_ASSERT(sector  < nr_sectors);
 	D_ASSERT(esector < nr_sectors);
@@ -839,53 +741,33 @@ STATIC int drbd_may_do_local_read(struct drbd_conf *mdev, sector_t sector, int s
 	sbnr = BM_SECT_TO_BIT(sector);
 	ebnr = BM_SECT_TO_BIT(esector);
 
-	return (0 == drbd_bm_count_bits(mdev, sbnr, ebnr));
+	return 0 == drbd_bm_count_bits(mdev, sbnr, ebnr);
 }
 
-/*
- * general note:
- * looking at the state (conn, disk, susp, pdsk) outside of the spinlock that
- * protects the state changes is inherently racy.
- *
- * FIXME verify this rationale why we may do so anyways:
- *
- * I think it "should" be like this:
- * as soon as we have a "ap_bio_cnt" reference we may test for "bad" states,
- * because the transition from "bad" to "good" states may only happen while no
- * application request is on the fly, so once we are positive about a "bad"
- * state, we know it won't get better during the lifetime of this request.
- *
- * In case we think we are ok, but "asynchronously" some interrupt or other
- * thread marks some operation as impossible, we are still ok, since we would
- * just try anyways, and then see that it does not work there and then.
- */
-
 STATIC int drbd_make_request_common(struct drbd_conf *mdev, struct bio *bio)
 {
 	const int rw = bio_rw(bio);
 	const int size = bio->bi_size;
 	const sector_t sector = bio->bi_sector;
-	struct drbd_barrier *b = NULL;
+	struct drbd_tl_epoch *b = NULL;
 	struct drbd_request *req;
 	int local, remote;
 	int err = -EIO;
 
-	/* allocate outside of all locks; get a "reference count" (ap_bio_cnt)
-	 * to avoid races with the disconnect/reconnect code.  */
-	inc_ap_bio(mdev);
+	/* allocate outside of all locks; */
 	req = drbd_req_new(mdev, bio);
 	if (!req) {
 		dec_ap_bio(mdev);
 		/* only pass the error to the upper layers.
 		 * if user cannot handle io errors, thats not our business. */
-		ERR("could not kmalloc() req\n");
+		dev_err(DEV, "could not kmalloc() req\n");
 		bio_endio(bio, -ENOMEM);
 		return 0;
 	}
 
-	dump_bio(mdev, bio, 0);
+	trace_drbd_bio(mdev, "Rq", bio, 0, req);
 
-	local = inc_local(mdev);
+	local = get_ldev(mdev);
 	if (!local) {
 		bio_put(req->private_bio); /* or we get a bio leak */
 		req->private_bio = NULL;
@@ -901,40 +783,24 @@ STATIC int drbd_make_request_common(struct drbd_conf *mdev, struct bio *bio)
 				 * it, then continue locally.
 				 * Or just issue the request remotely.
 				 */
-				/* FIXME
-				 * I think we have a RACE here. We request
-				 * something from the peer, then later some
-				 * write starts ...  and finished *before*
-				 * the answer to the read comes in, because
-				 * the ACK for the WRITE goes over
-				 * meta-socket ...
-				 * Maybe we need to properly lock reads
-				 * against the syncer, too. But if we have
-				 * some user issuing writes on an area that
-				 * he has pending reads on, _he_ is really
-				 * broke anyways, and would get "undefined
-				 * results" on _any_ io stack, even just the
-				 * local io stack.
-				 */
-
 				local = 0;
 				bio_put(req->private_bio);
 				req->private_bio = NULL;
-				dec_local(mdev);
+				put_ldev(mdev);
 			}
 		}
-		remote = !local && mdev->state.pdsk >= UpToDate;
+		remote = !local && mdev->state.pdsk >= D_UP_TO_DATE;
 	}
 
 	/* If we have a disk, but a READA request is mapped to remote,
-	 * we are Primary, Inconsistent, SyncTarget.
+	 * we are R_PRIMARY, D_INCONSISTENT, SyncTarget.
 	 * Just fail that READA request right here.
 	 *
 	 * THINK: maybe fail all READA when not local?
 	 *        or make this configurable...
 	 *        if network is slow, READA won't do any good.
 	 */
-	if (rw == READA && mdev->state.disk >= Inconsistent && !local) {
+	if (rw == READA && mdev->state.disk >= D_INCONSISTENT && !local) {
 		err = -EWOULDBLOCK;
 		goto fail_and_free_req;
 	}
@@ -947,56 +813,55 @@ STATIC int drbd_make_request_common(struct drbd_conf *mdev, struct bio *bio)
 	if (rw == WRITE && local)
 		drbd_al_begin_io(mdev, sector);
 
-	remote = remote && (mdev->state.pdsk == UpToDate ||
-			    ( mdev->state.pdsk == Inconsistent &&
-			      mdev->state.conn >= Connected ) );
+	remote = remote && (mdev->state.pdsk == D_UP_TO_DATE ||
+			    (mdev->state.pdsk == D_INCONSISTENT &&
+			     mdev->state.conn >= C_CONNECTED));
 
 	if (!(local || remote)) {
-		ERR("IO ERROR: neither local nor remote disk\n");
-		goto fail_and_free_req;
+		dev_err(DEV, "IO ERROR: neither local nor remote disk\n");
+		goto fail_free_complete;
 	}
 
 	/* For WRITE request, we have to make sure that we have an
-	 * unused_spare_barrier, in case we need to start a new epoch.
+	 * unused_spare_tle, in case we need to start a new epoch.
 	 * I try to be smart and avoid to pre-allocate always "just in case",
 	 * but there is a race between testing the bit and pointer outside the
 	 * spinlock, and grabbing the spinlock.
 	 * if we lost that race, we retry.  */
 	if (rw == WRITE && remote &&
-	    mdev->unused_spare_barrier == NULL &&
+	    mdev->unused_spare_tle == NULL &&
 	    test_bit(CREATE_BARRIER, &mdev->flags)) {
 allocate_barrier:
-		b = kmalloc(sizeof(struct drbd_barrier), GFP_NOIO);
+		b = kmalloc(sizeof(struct drbd_tl_epoch), GFP_NOIO);
 		if (!b) {
-			ERR("Failed to alloc barrier.\n");
+			dev_err(DEV, "Failed to alloc barrier.\n");
 			err = -ENOMEM;
-			goto fail_and_free_req;
+			goto fail_free_complete;
 		}
 	}
 
 	/* GOOD, everything prepared, grab the spin_lock */
 	spin_lock_irq(&mdev->req_lock);
 
-	/* FIXME race with drbd_disconnect and tl_clear? */
 	if (remote) {
-		remote = (mdev->state.pdsk == UpToDate ||
-			    ( mdev->state.pdsk == Inconsistent &&
-			      mdev->state.conn >= Connected ) );
+		remote = (mdev->state.pdsk == D_UP_TO_DATE ||
+			    (mdev->state.pdsk == D_INCONSISTENT &&
+			     mdev->state.conn >= C_CONNECTED));
 		if (!remote)
-			WARN("lost connection while grabbing the req_lock!\n");
+			dev_warn(DEV, "lost connection while grabbing the req_lock!\n");
 		if (!(local || remote)) {
-			ERR("IO ERROR: neither local nor remote disk\n");
+			dev_err(DEV, "IO ERROR: neither local nor remote disk\n");
 			spin_unlock_irq(&mdev->req_lock);
-			goto fail_and_free_req;
+			goto fail_free_complete;
 		}
 	}
 
-	if (b && mdev->unused_spare_barrier == NULL) {
-		mdev->unused_spare_barrier = b;
+	if (b && mdev->unused_spare_tle == NULL) {
+		mdev->unused_spare_tle = b;
 		b = NULL;
 	}
 	if (rw == WRITE && remote &&
-	    mdev->unused_spare_barrier == NULL &&
+	    mdev->unused_spare_tle == NULL &&
 	    test_bit(CREATE_BARRIER, &mdev->flags)) {
 		/* someone closed the current epoch
 		 * while we were grabbing the spinlock */
@@ -1018,10 +883,10 @@ allocate_barrier:
 	 * barrier packet.  To get the write ordering right, we only have to
 	 * make sure that, if this is a write request and it triggered a
 	 * barrier packet, this request is queued within the same spinlock. */
-	if (remote && mdev->unused_spare_barrier &&
-            test_and_clear_bit(CREATE_BARRIER, &mdev->flags)) {
-		_tl_add_barrier(mdev, mdev->unused_spare_barrier);
-		mdev->unused_spare_barrier = NULL;
+	if (remote && mdev->unused_spare_tle &&
+	    test_and_clear_bit(CREATE_BARRIER, &mdev->flags)) {
+		_tl_add_barrier(mdev, mdev->unused_spare_tle);
+		mdev->unused_spare_tle = NULL;
 	} else {
 		D_ASSERT(!(remote && rw == WRITE &&
 			   test_bit(CREATE_BARRIER, &mdev->flags)));
@@ -1061,7 +926,7 @@ allocate_barrier:
 			bio_put(req->private_bio);
 			req->private_bio = NULL;
 			drbd_al_complete_io(mdev, req->sector);
-			dec_local(mdev);
+			put_ldev(mdev);
 			local = 0;
 		}
 		if (remote)
@@ -1078,7 +943,7 @@ allocate_barrier:
 	/* NOTE remote first: to get the concurrent write detection right,
 	 * we must register the request before start of local IO.  */
 	if (remote) {
-		/* either WRITE and Connected,
+		/* either WRITE and C_CONNECTED,
 		 * or READ, and no local disk,
 		 * or READ, but not in sync.
 		 */
@@ -1091,13 +956,13 @@ allocate_barrier:
 	kfree(b); /* if someone else has beaten us to it... */
 
 	if (local) {
-		req->private_bio->bi_bdev = mdev->bc->backing_bdev;
+		req->private_bio->bi_bdev = mdev->ldev->backing_bdev;
 
-		dump_internal_bio("Pri", mdev, req->private_bio, 0);
+		trace_drbd_bio(mdev, "Pri", req->private_bio, 0, NULL);
 
-		if (FAULT_ACTIVE(mdev, rw==WRITE ? DRBD_FAULT_DT_WR :
-				       (rw==READ ? DRBD_FAULT_DT_RD :
-				                   DRBD_FAULT_DT_RA) ))
+		if (FAULT_ACTIVE(mdev, rw == WRITE ? DRBD_FAULT_DT_WR
+				     : rw == READ  ? DRBD_FAULT_DT_RD
+				     :               DRBD_FAULT_DT_RA))
 			bio_endio(req->private_bio, -EIO);
 		else
 			generic_make_request(req->private_bio);
@@ -1109,10 +974,20 @@ allocate_barrier:
 
 	return 0;
 
+fail_free_complete:
+	if (rw == WRITE && local)
+		drbd_al_complete_io(mdev, sector);
 fail_and_free_req:
-	kfree(b);
+	if (local) {
+		bio_put(req->private_bio);
+		req->private_bio = NULL;
+		put_ldev(mdev);
+	}
 	bio_endio(bio, err);
 	drbd_req_free(req);
+	dec_ap_bio(mdev);
+	kfree(b);
+
 	return 0;
 }
 
@@ -1124,14 +999,14 @@ fail_and_free_req:
 static int drbd_fail_request_early(struct drbd_conf *mdev, int is_write)
 {
 	/* Unconfigured */
-	if (mdev->state.conn == Disconnecting &&
-	    mdev->state.disk == Diskless)
+	if (mdev->state.conn == C_DISCONNECTING &&
+	    mdev->state.disk == D_DISKLESS)
 		return 1;
 
-	if (mdev->state.role != Primary &&
-		( !allow_oos || is_write) ) {
+	if (mdev->state.role != R_PRIMARY &&
+		(!allow_oos || is_write)) {
 		if (DRBD_ratelimit(5*HZ, 5)) {
-			ERR("Process %s[%u] tried to %s; "
+			dev_err(DEV, "Process %s[%u] tried to %s; "
 			    "since we are not in Primary state, "
 			    "we cannot allow this\n",
 			    current->comm, current->pid,
@@ -1149,13 +1024,9 @@ static int drbd_fail_request_early(struct drbd_conf *mdev, int is_write)
 	 * to serialize state changes, this is racy, since we may lose
 	 * the connection *after* we test for the cstate.
 	 */
-	if ( mdev->state.disk < UpToDate &&
-	     mdev->state.conn < Connected) {
+	if (mdev->state.disk < D_UP_TO_DATE && mdev->state.pdsk < D_UP_TO_DATE) {
 		if (DRBD_ratelimit(5*HZ, 5))
-			ERR("Sorry, I have no access to good data anymore.\n");
-		/*
-		 * FIXME suspend, loop waiting on cstate wait?
-		 */
+			dev_err(DEV, "Sorry, I have no access to good data anymore.\n");
 		return 1;
 	}
 
@@ -1181,7 +1052,7 @@ int drbd_make_request_26(struct request_queue *q, struct bio *bio)
 	 * i.e. in drbd_init_set_defaults we set the NO_BARRIER_SUPP bit.
 	 */
 	if (unlikely(bio_barrier(bio) && test_bit(NO_BARRIER_SUPP, &mdev->flags))) {
-		/* WARN("Rejecting barrier request as underlying device does not support\n"); */
+		/* dev_warn(DEV, "Rejecting barrier request as underlying device does not support\n"); */
 		bio_endio(bio, -EOPNOTSUPP);
 		return 0;
 	}
@@ -1190,7 +1061,7 @@ int drbd_make_request_26(struct request_queue *q, struct bio *bio)
 	 * what we "blindly" assume:
 	 */
 	D_ASSERT(bio->bi_size > 0);
-	D_ASSERT( (bio->bi_size & 0x1ff) == 0);
+	D_ASSERT((bio->bi_size & 0x1ff) == 0);
 	D_ASSERT(bio->bi_idx == 0);
 
 	/* to make some things easier, force allignment of requests within the
@@ -1198,15 +1069,20 @@ int drbd_make_request_26(struct request_queue *q, struct bio *bio)
 	s_enr = bio->bi_sector >> HT_SHIFT;
 	e_enr = (bio->bi_sector+(bio->bi_size>>9)-1) >> HT_SHIFT;
 
-	if (unlikely(s_enr != e_enr)) {
-	if (bio->bi_vcnt != 1 || bio->bi_idx != 0) {
+	if (likely(s_enr == e_enr)) {
+		inc_ap_bio(mdev, 1);
+		return drbd_make_request_common(mdev, bio);
+	}
+
+	/* can this bio be split generically?
+	 * Maybe add our own split-arbitrary-bios function. */
+	if (bio->bi_vcnt != 1 || bio->bi_idx != 0 || bio->bi_size > DRBD_MAX_SEGMENT_SIZE) {
 		/* rather error out here than BUG in bio_split */
-		ERR("bio would need to, but cannot, be split: "
+		dev_err(DEV, "bio would need to, but cannot, be split: "
 		    "(vcnt=%u,idx=%u,size=%u,sector=%llu)\n",
 		    bio->bi_vcnt, bio->bi_idx, bio->bi_size,
 		    (unsigned long long)bio->bi_sector);
 		bio_endio(bio, -EINVAL);
-		return 0;
 	} else {
 		/* This bio crosses some boundary, so we have to split it. */
 		struct bio_pair *bp;
@@ -1219,18 +1095,29 @@ int drbd_make_request_26(struct request_queue *q, struct bio *bio)
 		 * first_sectors = 64 - (262269 & 63) = 3
 		 */
 		const sector_t sect = bio->bi_sector;
-		const int sps = 1<<HT_SHIFT; /* sectors per slot */
-		const int mask = sps -1;
+		const int sps = 1 << HT_SHIFT; /* sectors per slot */
+		const int mask = sps - 1;
 		const sector_t first_sectors = sps - (sect & mask);
-		bp = bio_split(bio, bio_split_pool, first_sectors);
-		drbd_make_request_26(q, &bp->bio1);
-		drbd_make_request_26(q, &bp->bio2);
-		bio_pair_release(bp);
-		return 0;
-	}
-	}
+		bp = bio_split(bio,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
+				bio_split_pool,
+#endif
+				first_sectors);
 
-	return drbd_make_request_common(mdev,bio);
+		/* we need to get a "reference count" (ap_bio_cnt)
+		 * to avoid races with the disconnect/reconnect/suspend code.
+		 * In case we need to split the bio here, we need to get two references
+		 * atomically, otherwise we might deadlock when trying to submit the
+		 * second one! */
+		inc_ap_bio(mdev, 2);
+
+		D_ASSERT(e_enr == s_enr + 1);
+
+		drbd_make_request_common(mdev, &bp->bio1);
+		drbd_make_request_common(mdev, &bp->bio2);
+		bio_pair_release(bp);
+	}
+	return 0;
 }
 
 /* This is called by bio_add_page().  With this function we reduce
@@ -1246,12 +1133,18 @@ int drbd_make_request_26(struct request_queue *q, struct bio *bio)
  * cross extent boundaries.  those are dealt with (bio_split) in
  * drbd_make_request_26.
  */
-int drbd_merge_bvec(struct request_queue *q, struct bio *bio, struct bio_vec *bvec)
+int drbd_merge_bvec(struct request_queue *q,
+#ifdef HAVE_bvec_merge_data
+		struct bvec_merge_data *bvm,
+#else
+		struct bio *bvm,
+#endif
+		struct bio_vec *bvec)
 {
 	struct drbd_conf *mdev = (struct drbd_conf *) q->queuedata;
 	unsigned int bio_offset =
-		(unsigned int)bio->bi_sector << 9; /* 32 bit */
-	unsigned int bio_size = bio->bi_size;
+		(unsigned int)bvm->bi_sector << 9; /* 32 bit */
+	unsigned int bio_size = bvm->bi_size;
 	int limit, backing_limit;
 
 	limit = DRBD_MAX_SEGMENT_SIZE
@@ -1261,14 +1154,14 @@ int drbd_merge_bvec(struct request_queue *q, struct bio *bio, struct bio_vec *bv
 	if (bio_size == 0) {
 		if (limit <= bvec->bv_len)
 			limit = bvec->bv_len;
-	} else if (limit && inc_local(mdev)) {
+	} else if (limit && get_ldev(mdev)) {
 		struct request_queue * const b =
-			mdev->bc->backing_bdev->bd_disk->queue;
-		if (b->merge_bvec_fn && mdev->bc->dc.use_bmbv) {
-			backing_limit = b->merge_bvec_fn(b, bio, bvec);
+			mdev->ldev->backing_bdev->bd_disk->queue;
+		if (b->merge_bvec_fn && mdev->ldev->dc.use_bmbv) {
+			backing_limit = b->merge_bvec_fn(b, bvm, bvec);
 			limit = min(limit, backing_limit);
 		}
-		dec_local(mdev);
+		put_ldev(mdev);
 	}
 	return limit;
 }
