@@ -1,10 +1,18 @@
+#ifndef _DRBD_WRAPPERS_H
+#define _DRBD_WRAPPERS_H
+
 #include <linux/ctype.h>
+#include <linux/net.h>
 
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 # error "use a 2.6 kernel, please"
 #endif
 
+#include <linux/slab.h>
+
+/* for the proc_create wrapper */
+#include <linux/proc_fs.h>
 
 /* struct page has a union in 2.6.15 ...
  * an anonymous union and struct since 2.6.16
@@ -15,14 +23,47 @@
 # define set_page_private(page, v)	((page)->private = (v))
 #endif
 
+/* mutex was not available before 2.6.16.
+ * various vendors provide various degrees of backports.
+ * we provide the missing parts ourselves, if neccessary.
+ * this one is for RHEL/Centos 4 */
+#if defined(mutex_lock) && !defined(mutex_is_locked)
+#define mutex_is_locked(m) (atomic_read(&(m)->count) != 1)
+#endif
+
 /* see get_sb_bdev and bd_claim */
 extern char *drbd_sec_holder;
 
-static inline sector_t drbd_get_hardsect(struct block_device *bdev)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,31)
+static inline unsigned short queue_logical_block_size(struct request_queue *q)
 {
-	return bdev->bd_disk->queue->hardsect_size;
+	int retval = 512;
+	if (q && q->hardsect_size)
+		retval = q->hardsect_size;
+	return retval;
 }
 
+static inline sector_t bdev_logical_block_size(struct block_device *bdev)
+{
+	return queue_logical_block_size(bdev_get_queue(bdev));
+}
+
+static inline unsigned int queue_max_segment_size(struct request_queue *q)
+{
+	return q->max_segment_size;
+}
+
+static inline unsigned int queue_max_sectors(struct request_queue *q)
+{
+	return q->max_sectors;
+}
+
+static inline void blk_queue_logical_block_size(struct request_queue *q, unsigned short size)
+{
+	q->hardsect_size = size;
+}
+#endif
+
 /* Returns the number of 512 byte sectors of the device */
 static inline sector_t drbd_get_capacity(struct block_device *bdev)
 {
@@ -47,7 +88,8 @@ static inline int drbd_bio_has_active_page(struct bio *bio)
 	int i;
 
 	__bio_for_each_segment(bvec, bio, i, 0) {
-		if (page_count(bvec->bv_page) > 1) return 1;
+		if (page_count(bvec->bv_page) > 1)
+			return 1;
 	}
 
 	return 0;
@@ -64,11 +106,11 @@ static inline int drbd_bio_has_active_page(struct bio *bio)
 #else
 #define BIO_ENDIO_TYPE void
 #define BIO_ENDIO_ARGS(b,e) (b,e)
-#define BIO_ENDIO_FN_START while(0) {}
+#define BIO_ENDIO_FN_START do {} while (0)
 #define BIO_ENDIO_FN_RETURN return
 #endif
 
-// bi_end_io handlers
+/* bi_end_io handlers */
 extern BIO_ENDIO_TYPE drbd_md_io_complete BIO_ENDIO_ARGS(struct bio *bio, int error);
 extern BIO_ENDIO_TYPE drbd_endio_read_sec BIO_ENDIO_ARGS(struct bio *bio, int error);
 extern BIO_ENDIO_TYPE drbd_endio_write_sec BIO_ENDIO_ARGS(struct bio *bio, int error);
@@ -80,6 +122,11 @@ extern BIO_ENDIO_TYPE drbd_endio_pri BIO_ENDIO_ARGS(struct bio *bio, int error);
 #define kmem_cache_create(N,S,A,F,C) kmem_cache_create(N,S,A,F,C,NULL)
 #endif
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+# undef HAVE_bvec_merge_data
+# define HAVE_bvec_merge_data 1
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
 static inline void sg_set_page(struct scatterlist *sg, struct page *page,
 			       unsigned int len, unsigned int offset)
@@ -101,6 +148,40 @@ static inline void sg_set_buf(struct scatterlist *sg, const void *buf,
 
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+# define BD_OPS_USE_FMODE
+#endif
+
+/* how to get to the kobj of a gendisk.
+ * see also upstream commits
+ * edfaa7c36574f1bf09c65ad602412db9da5f96bf
+ * ed9e1982347b36573cd622ee5f4e2a7ccd79b3fd
+ * 548b10eb2959c96cef6fc29fc96e0931eeb53bc5
+ */
+#ifndef dev_to_disk
+# define disk_to_kobj(disk) (&(disk)->kobj)
+#else
+# ifndef disk_to_dev
+#  define disk_to_dev(disk) (&(disk)->dev)
+# endif
+# define disk_to_kobj(disk) (&disk_to_dev(disk)->kobj)
+#endif
+static inline void drbd_kobject_uevent(struct drbd_conf *mdev)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,15)
+	kobject_uevent(disk_to_kobj(mdev->vdisk), KOBJ_CHANGE, NULL);
+#else
+	kobject_uevent(disk_to_kobj(mdev->vdisk), KOBJ_CHANGE);
+	/* rhel4 / sles9 and older don't have this at all,
+	 * which means user space (udev) won't get events about possible changes of
+	 * corresponding resource + disk names after the initial drbd minor creation.
+	 */
+#endif
+#endif
+}
+
+
 /*
  * used to submit our private bio
  */
@@ -149,10 +230,22 @@ static inline void drbd_plug_device(struct drbd_conf *mdev)
 #define kmem_cache kmem_cache_s
 #endif
 
+#ifdef DEFINE_KERNEL_SOCK_SHUTDOWN
+enum sock_shutdown_cmd {
+	SHUT_RD = 0,
+	SHUT_WR = 1,
+	SHUT_RDWR = 2,
+};
+static inline int kernel_sock_shutdown(struct socket *sock, enum sock_shutdown_cmd how)
+{
+	return sock->ops->shutdown(sock, how);
+}
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,23)
 static inline void drbd_unregister_blkdev(unsigned int major, const char *name)
 {
-	int ret = unregister_blkdev(major,name);
+	int ret = unregister_blkdev(major, name);
 	if (ret)
 		printk(KERN_ERR "drbd: unregister of device failed\n");
 }
@@ -176,7 +269,7 @@ static __inline__ int atomic_add_return(int i, atomic_t *v)
 
 static __inline__ int atomic_sub_return(int i, atomic_t *v)
 {
-	return atomic_add_return(-i,v);
+	return atomic_add_return(-i, v);
 }
 
 #define atomic_inc_return(v)  (atomic_add_return(1,v))
@@ -212,7 +305,7 @@ no_xadd: /* Legacy 386 processor */
 
 static __inline__ int atomic_sub_return(int i, atomic_t *v)
 {
-	return atomic_add_return(-i,v);
+	return atomic_add_return(-i, v);
 }
 
 #define atomic_inc_return(v)  (atomic_add_return(1,v))
@@ -251,11 +344,12 @@ crypto_alloc_hash(char *alg_name, u32 type, u32 mask)
 	struct crypto_hash *ch;
 	char *closing_bracket;
 
-	// "hmac(xxx)" is in alg_name we need that xxx.
-	closing_bracket = strchr(alg_name,')');
-	if(!closing_bracket) {
-		ch = kmalloc(sizeof(struct crypto_hash),GFP_KERNEL);
-		if(!ch) return ERR_PTR(-ENOMEM);
+	/* "hmac(xxx)" is in alg_name we need that xxx. */
+	closing_bracket = strchr(alg_name, ')');
+	if (!closing_bracket) {
+		ch = kmalloc(sizeof(struct crypto_hash), GFP_KERNEL);
+		if (!ch)
+			return ERR_PTR(-ENOMEM);
 		ch->base = crypto_alloc_tfm(alg_name, 0);
 		if (ch->base == NULL) {
 			kfree(ch);
@@ -263,10 +357,12 @@ crypto_alloc_hash(char *alg_name, u32 type, u32 mask)
 		}
 		return ch;
 	}
-	if(closing_bracket-alg_name < 6) return ERR_PTR(-ENOENT);
+	if (closing_bracket-alg_name < 6)
+		return ERR_PTR(-ENOENT);
 
-	ch = kmalloc(sizeof(struct crypto_hash),GFP_KERNEL);
-	if(!ch) return ERR_PTR(-ENOMEM);
+	ch = kmalloc(sizeof(struct crypto_hash), GFP_KERNEL);
+	if (!ch)
+		return ERR_PTR(-ENOMEM);
 
 	*closing_bracket = 0;
 	ch->base = crypto_alloc_tfm(alg_name + 5, 0);
@@ -281,7 +377,7 @@ crypto_alloc_hash(char *alg_name, u32 type, u32 mask)
 }
 
 static inline int
-crypto_hash_setkey(struct crypto_hash *hash,const u8 *key,unsigned int keylen)
+crypto_hash_setkey(struct crypto_hash *hash, const u8 *key, unsigned int keylen)
 {
 	hash->key = key;
 	hash->keylen = keylen;
@@ -294,7 +390,7 @@ crypto_hash_digest(struct hash_desc *desc, struct scatterlist *sg,
 		   unsigned int nbytes, u8 *out)
 {
 
-	crypto_hmac(desc->tfm->base, (u8*)desc->tfm->key,
+	crypto_hmac(desc->tfm->base, (u8 *)desc->tfm->key,
 		    &desc->tfm->keylen, sg, 1 /* ! */ , out);
 	/* ! this is not generic. Would need to convert nbytes -> nsg */
 
@@ -303,8 +399,9 @@ crypto_hash_digest(struct hash_desc *desc, struct scatterlist *sg,
 
 static inline void crypto_free_hash(struct crypto_hash *tfm)
 {
-	if (!tfm) return;
-	crypto_free_tfm(tfm->base); 
+	if (!tfm)
+		return;
+	crypto_free_tfm(tfm->base);
 	kfree(tfm);
 }
 
@@ -342,11 +439,24 @@ static inline int crypto_hash_final(struct hash_desc *desc, u8 *out)
 
 #endif
 
+static inline int drbd_crypto_is_hash(struct crypto_tfm *tfm)
+{
+#ifdef CRYPTO_ALG_TYPE_HASH_MASK
+	/* see include/linux/crypto.h */
+	return !((crypto_tfm_alg_type(tfm) ^ CRYPTO_ALG_TYPE_HASH)
+		& CRYPTO_ALG_TYPE_HASH_MASK);
+#else
+	return crypto_tfm_alg_type(tfm) == CRYPTO_ALG_TYPE_HASH;
+#endif
+}
+
+
 #ifdef NEED_BACKPORT_OF_KZALLOC
 static inline void *kzalloc(size_t size, int flags)
 {
-	void *rv = kmalloc(size,flags);
-	if(rv) memset(rv,0,size);
+	void *rv = kmalloc(size, flags);
+	if (rv)
+		memset(rv, 0, size);
 
 	return rv;
 }
@@ -440,3 +550,67 @@ static inline int backport_bitmap_parse(const char *buf, unsigned int buflen,
 #define KERNEL_HAS_GFP_T
 typedef unsigned gfp_t;
 #endif
+
+
+/* struct kvec didn't exist before 2.6.8, this is an ugly
+ * #define to work around it ... - jt */
+
+#ifndef KERNEL_HAS_KVEC
+#define kvec iovec
+#endif
+
+#ifndef net_random
+#define random32 net_random
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,30)
+#define BDI_async_congested BDI_write_congested
+#define BDI_sync_congested  BDI_read_congested
+#endif
+
+/* see upstream commits
+ * 2d3a4e3666325a9709cc8ea2e88151394e8f20fc (in 2.6.25-rc1)
+ * 59b7435149eab2dd06dd678742faff6049cb655f (in 2.6.26-rc1)
+ * this "backport" does not close the race that lead to the API change,
+ * but only provides an equivalent function call.
+ */
+#ifndef KERNEL_HAS_PROC_CREATE
+static inline struct proc_dir_entry *proc_create(const char *name,
+	mode_t mode, struct proc_dir_entry *parent,
+	struct file_operations *proc_fops)
+{
+	struct proc_dir_entry *pde = create_proc_entry(name, mode, parent);
+	if (pde)
+		pde->proc_fops = proc_fops;
+	return pde;
+}
+
+#endif
+
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,30)
+#define TP_PROTO(args...)	args
+#define TP_ARGS(args...)		args
+
+#undef DECLARE_TRACE
+#define DECLARE_TRACE(name, proto, args)				\
+	static inline void _do_trace_##name(struct tracepoint *tp, proto) \
+	{ }								\
+	static inline void trace_##name(proto)				\
+	{ }								\
+	static inline int register_trace_##name(void (*probe)(proto))	\
+	{								\
+		return -ENOSYS;						\
+	}								\
+	static inline int unregister_trace_##name(void (*probe)(proto))	\
+	{								\
+		return -ENOSYS;						\
+	}
+
+#undef DEFINE_TRACE
+#define DEFINE_TRACE(name)
+
+#endif
+
+
+#endif
