@@ -22,9 +22,10 @@
 
 extern const char *drbd_buildtag(void);
 
-#define REL_VERSION "8.0.14"
-#define API_VERSION 86
-#define PRO_VERSION 86
+#define REL_VERSION "8.3.2"
+#define API_VERSION 88
+#define PRO_VERSION_MIN 86
+#define PRO_VERSION_MAX 90
 
 #ifndef __CHECKER__   /* for a sparse run, we need all STATICs */
 #define DBG_ALL_SYMBOLS /* no static functs, improves quality of OOPS traces */
@@ -41,19 +42,6 @@ extern const char *drbd_buildtag(void);
 #define PARANOIA
  */
 
-/* Dump every hour the usage / not usage of zero copy IO */
-/* #define SHOW_SENDPAGE_USAGE */
-
-/* Define this to enable dynamic tracing controlled by module parameters
- * at run time. This enables ALL use of dynamic tracing including packet
- * and bio dumping, etc */
-#define ENABLE_DYNAMIC_TRACE
-
-/* You can disable the use of the sendpage() call (= zero copy IO)
- * If you have the feeling that this might be the cause for troubles.
-#define DRBD_DISABLE_SENDPAGE
- */
-
 /* Enable fault insertion code */
 #define DRBD_ENABLE_FAULTS
 
@@ -74,6 +62,10 @@ extern const char *drbd_buildtag(void);
  * some older distribution kernels don't. */
 //#define DEFINE_SOCK_CREATE_KERN
 
+/* 2.6.24 and later have kernel_sock_shutdown.
+ * some older distribution kernels may also have a backport. */
+//#define DEFINE_KERNEL_SOCK_SHUTDOWN
+
 /* in older kernels (vanilla < 2.6.16) struct netlink_skb_parms has a
  * member called dst_groups. Later it is called dst_group (without 's'). */
 //#define DRBD_NL_DST_GROUPS
@@ -85,4 +77,22 @@ extern const char *drbd_buildtag(void);
 //#define NEED_SG_SET_BUF
 #define HAVE_LINUX_SCATTERLIST_H
 
+/* 2.6.29 and up no longer have swabb.h */
+//#define HAVE_LINUX_BYTEORDER_SWABB_H
+
+/* Some vendor kernels < 2.6.7 might define msleep in one or
+ * another way .. */
+
+#define KERNEL_HAS_MSLEEP
+
+/* Some other kernels < 2.6.8 do not have struct kvec,
+ * others do.. */
+
+#define KERNEL_HAS_KVEC
+
+/* Actually availabe since 2.6.25, but venders have backported...
+ */
+#define KERNEL_HAS_PROC_CREATE
+
+
 #endif
