@@ -29,7 +29,8 @@ KDIR=${KDIR%/}
 if test -z "$O"; then
 	## just in case...
 	## detect if $KDIR points to something which is actually $O ...
-	X=$( make help | sed -ne '/ -C .* O=.* help$/p' | tr -s ' ' )
+	X=$( make no-such-makefile-target 2>/dev/null |
+	     sed -ne '/ -C .* O=.* no-such-makefile-target$/p' | tr -s ' ' )
 	if [[ -n $X ]]; then
 		KDIR=${X##* -C }; KDIR=${KDIR%% *}; KDIR=$(cd $KDIR && pwd)
 		O=${X##* O=}; O=${O%% *}; O=$(cd $KDIR && cd $O && pwd)
@@ -48,6 +49,7 @@ $O/{.config,Makefile,include/linux/version.h}
 test -e $O/include/asm/atomic.h  ||
 test -e $O/include/asm/arch/atomic.h  ||
 test -e $O/include2/asm/atomic.h ||
+test -e $KDIR/include/asm-generic/atomic.h ||
 exit 1
 
 if grep_q "^PATCHLEVEL *= *6" $KDIR/Makefile ; then
@@ -66,7 +68,8 @@ if grep_q "^PATCHLEVEL *= *6" $KDIR/Makefile ; then
     $O/include2/asm/atomic.h \
     $O/include/asm/atomic_32.h \
     $O/include2/asm/atomic_32.h \
-    $O/include/asm/arch/atomic_32.h
+    $O/include/asm/arch/atomic_32.h \
+    $KDIR/include/asm-generic/atomic.h
   do
     if grep_q "atomic_add_return" $f; then
       have_atomic_add=1
@@ -83,6 +86,11 @@ if grep_q "^PATCHLEVEL *= *6" $KDIR/Makefile ; then
   else
     have_sock_create_kern=0
   fi
+  if grep_q "kernel_sock_shutdown" $KDIR/include/linux/net.h ; then
+    have_kernel_sock_shutdown=1
+  else
+    have_kernel_sock_shutdown=0
+  fi
   if grep_q "dst_groups" $KDIR/include/linux/netlink.h ; then
     have_nl_dst_groups=1
   else
@@ -114,6 +122,16 @@ if grep_q "^PATCHLEVEL *= *6" $KDIR/Makefile ; then
   else
     have_kvec=0
   fi
+  if test -e $KDIR/include/linux/byteorder/swabb.h ; then
+    have_linux_byteorder_swabb_h=1
+  else
+    have_linux_byteorder_swabb_h=0
+  fi
+  if grep_q "proc_create(" $KDIR/include/linux/proc_fs.h ; then
+    have_proc_create=1
+  else
+    have_proc_create=0
+  fi
 else
     # not a 2.6. kernel. just leave it alone...
     exit 0
@@ -133,6 +151,8 @@ perl -pe "
   { ( $have_kmem_cache_s ? '' : '//' ) . \$1}e;
  s{.*(#define DEFINE_SOCK_CREATE_KERN.*)}
   { ( $have_sock_create_kern ? '//' : '' ) . \$1}e;
+ s{.*(#define DEFINE_KERNEL_SOCK_SHUTDOWN.*)}
+  { ( $have_kernel_sock_shutdown ? '//' : '' ) . \$1}e;
  s{.*(#define DRBD_NL_DST_GROUPS.*)}
   { ( $have_nl_dst_groups ? '' : '//' ) . \$1}e;
  s{.*(#define NEED_BACKPORT_OF_KZALLOC.*)}
@@ -145,6 +165,10 @@ perl -pe "
   { ( $have_msleep ? '' : '//' ) . \$1}e;
  s{.*(#define KERNEL_HAS_KVEC.*)}
   { ( $have_kvec ? '' : '//' ) . \$1}e;
+ s{.*(#define HAVE_LINUX_BYTEORDER_SWABB_H.*)}
+  { ( $have_linux_byteorder_swabb_h ? '' : '//' ) . \$1}e;
+ s{.*(#define KERNEL_HAS_PROC_CREATE.*)}
+  { ( $have_proc_create ? '' : '//' ) . \$1}e;
  " \
 	  < ./linux/drbd_config.h \
 	  > ./linux/drbd_config.h.new
