@@ -24,10 +24,9 @@ enum {
   RETURN_PID           = 2,
   SLEEPS_FOREVER       = 4,
 
-  SUPRESS_STDERR       = 16,
-
-  RETURN_STDOUT_FD     = 32,
-
+  SUPRESS_STDERR       = 0x10,
+  RETURN_STDOUT_FD     = 0x20,
+  RETURN_STDERR_FD     = 0x40,
   DONT_REPORT_FAILED   = 0x80,
 };
 
@@ -87,6 +86,7 @@ struct d_host_info
 {
   struct d_name *on_hosts;
   char* device;
+  int device_minor;
   char* disk;
   char* address;
   char* port;
@@ -98,7 +98,9 @@ struct d_host_info
   struct d_proxy_info *proxy;
   struct d_host_info* next;
   struct d_resource* lower;  /* for device stacking */
+  char *lower_name;          /* for device stacking, before bind_stacked_res() */
   int config_line;
+  unsigned int by_address:1; /* Match to machines by address, not by names (=on_hosts) */
 };
 
 struct d_option
@@ -118,6 +120,7 @@ struct d_resource
 
   /* these get propagated to host_info sections later. */
   char* device;
+  int device_minor;
   char* disk;
   char* meta_disk;
   char* meta_index;
@@ -133,10 +136,12 @@ struct d_resource
   struct d_option* proxy_options;
   struct d_resource* next;
   struct d_name *become_primary_on;
+  char *config_file; /* The config file this resource is define in.*/
+  int start_line;
   unsigned int stacked_timeouts:1;
   unsigned int ignore:1;
-  unsigned int stacked:1;
-  int me_minor; /* cache local minor number of this resource */
+  unsigned int stacked:1;        /* Stacked on this node */
+  unsigned int stacked_on_one:1; /* Stacked either on me or on peer */
 };
 
 extern int adm_attach(struct d_resource* ,const char* );
@@ -148,18 +153,19 @@ extern int _admm_generic(struct d_resource* ,const char*, int flags);
 extern int m_system(char**, int, struct d_resource*);
 extern struct d_option* find_opt(struct d_option*,char*);
 extern void validate_resource(struct d_resource *);
-extern int vcheck_uniq(const char* what, const char *fmt, va_list ap);
-extern int check_uniq(const char* what, const char *fmt, ...);
-extern void verify_ips(struct d_resource* res);
 extern void schedule_dcmd( int (* function)(struct d_resource*,const char* ),
 			   struct d_resource* res,
 			   char* arg,
 			   int order);
 
+extern int version_code_kernel(void);
+extern int version_code_userland(void);
+extern void warn_on_version_mismatch(void);
 extern void uc_node(enum usage_count_type type);
 extern int adm_create_md(struct d_resource* res ,const char* cmd);
 extern void convert_discard_opt(struct d_resource* res);
 extern void convert_after_option(struct d_resource* res);
+extern int have_ip(const char *af, const char *ip);
 
 /* See drbdadm_minor_table.c */
 extern int register_minor(int minor, const char *path);
@@ -167,12 +173,14 @@ extern int unregister_minor(int minor);
 extern char *lookup_minor(int minor);
 
 enum pr_flags {
-  ThisHRequired = 1,
-  PeerHRequired = 2,
-  BothHRequired = ThisHRequired | PeerHRequired,
   NoneHAllowed  = 4,
+  IgnDiscardMyData = 8
+};
+enum pp_flags {
+	match_on_proxy = 1,
 };
 extern struct d_resource* parse_resource(char*, enum pr_flags);
+extern void post_parse(struct d_resource *config, enum pp_flags);
 extern struct d_option *new_opt(char *name, char *value);
 extern int name_in_names(char *name, struct d_name *names);
 extern char *_names_to_str(char* buffer, struct d_name *names);
@@ -180,22 +188,29 @@ extern char *_names_to_str_c(char* buffer, struct d_name *names, char c);
 #define NAMES_STR_SIZE 255
 #define names_to_str(N) _names_to_str(alloca(NAMES_STR_SIZE+1), N)
 #define names_to_str_c(N, C) _names_to_str_c(alloca(NAMES_STR_SIZE+1), N, C)
-void free_names(struct d_name *names);
+extern void free_names(struct d_name *names);
+extern void set_me_in_resource(struct d_resource* res, int match_on_proxy);
+extern void set_peer_in_resource(struct d_resource* res, int peer_required);
+extern void set_on_hosts_in_res(struct d_resource *res);
+extern void set_disk_in_res(struct d_resource *res);
 
 extern char* config_file;
 extern int config_valid;
 extern struct d_resource* config;
 extern struct d_resource* common;
 extern struct d_globals global_options;
-extern int line, fline, c_resource_start;
+extern int line, fline;
+extern struct hsearch_data global_htable;
 
 extern int no_tty;
 extern int dry_run;
+extern int verbose;
 extern char* drbdsetup;
 extern char ss_buffer[255];
 extern struct utsname nodeinfo;
 
 extern char* setup_opts[10];
+extern char* connect_to_host;
 extern int soi;
 
 
