@@ -134,7 +134,7 @@ static int addr_equal(struct d_resource* conf, struct d_resource* running)
 	int equal;
 
 	if (conf->peer == NULL && running->peer == NULL) return 1;
-	if (conf->peer == NULL || running->peer == NULL) return 0;
+	if (running->peer == NULL) return 0;
 
 	equal = !strcmp(conf->me->address,        running->me->address) &&
 		!strcmp(conf->me->port,           running->me->port) &&
@@ -146,7 +146,7 @@ static int addr_equal(struct d_resource* conf, struct d_resource* running)
 			!strcmp(conf->me->proxy->inside_port, running->peer->port) &&
 			!strcmp(conf->me->proxy->inside_af,   running->peer->address_family);
 	else
-		equal = equal &&
+		equal = equal && conf->peer &&
 			!strcmp(conf->peer->address,        running->peer->address) &&
 			!strcmp(conf->peer->port,           running->peer->port) &&
 			!strcmp(conf->peer->address_family, running->peer->address_family);
@@ -200,17 +200,23 @@ int adm_adjust(struct d_resource* res,char* unused __attribute((unused)))
 	argv[argc++]="show";
 	argv[argc++]=0;
 
+	/* disable check_uniq, so it won't interfere
+	 * with parsing of drbdsetup show output */
+	config_valid = 2;
+
 	yyin = m_popen(&pid,argv);
 	line = 1;
-	sprintf(config_file_dummy,"drbdsetup %s show", res->me->device);
+	sprintf(config_file_dummy,"drbdsetup %u show", res->me->device_minor);
 	config_file = config_file_dummy;
-	running = parse_resource(config_file_dummy, 0);
+	running = parse_resource(config_file_dummy, IgnDiscardMyData);
 	fclose(yyin);
 	waitpid(pid,0,0);
+	post_parse(running, 0);
+	set_peer_in_resource(running, 0);
 
 	do_attach  = !opts_equal(res->disk_options, running->disk_options);
 	if(running->me) {
-		do_attach |= strcmp(res->me->device, running->me->device);
+		do_attach |= (res->me->device_minor != running->me->device_minor);
 		do_attach |= !disk_equal(res->me, running->me);
 		have_disk = (running->me->disk != NULL);
 	} else  do_attach |= 1;
@@ -228,7 +234,8 @@ int adm_adjust(struct d_resource* res,char* unused __attribute((unused)))
 	}
 	if(do_syncer)  schedule_dcmd(adm_syncer,res,"syncer",1);
 	if(do_connect) {
-		if(have_net) schedule_dcmd(adm_generic_s,res,"disconnect",0);
+		if (have_net && res->peer)
+			schedule_dcmd(adm_generic_s,res,"disconnect",0);
 		schedule_dcmd(adm_connect,res,"connect",2);
 	}
 
