@@ -1,4 +1,5 @@
 /*
+ *
    drbdadm_parser.c a hand crafted parser
 
    This file is part of DRBD by Philipp Reisner and Lars Ellenberg.
@@ -28,6 +29,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <glob.h>
+#include <search.h>
 
 #include "drbdadm.h"
 #include "linux/drbd_limits.h"
@@ -39,6 +42,7 @@ YYSTYPE yylval;
 /////////////////////
 
 static int c_section_start;
+void my_parse(void);
 
 struct d_name *names_from_str(char* str)
 {
@@ -55,6 +59,9 @@ char *_names_to_str_c(char* buffer, struct d_name *names, char c)
 {
 	int n = 0;
 
+	if (!names)
+		return buffer;
+
 	while (1) {
 		n += snprintf(buffer + n, NAMES_STR_SIZE - n, "%s", names->name);
 		names = names->next;
@@ -185,6 +192,10 @@ void range_check(const enum range_checks what, const char *name,
 		m_strtoll_range(value, 1, name, DRBD_SNDBUF_SIZE_MIN,
 				DRBD_SNDBUF_SIZE_MAX);
 		break;
+	case R_RCVBUF_SIZE:
+		m_strtoll_range(value, 1, name, DRBD_RCVBUF_SIZE_MIN,
+				DRBD_RCVBUF_SIZE_MAX);
+		break;
 	case R_KO_COUNT:
 		m_strtoll_range(value, 1, name, DRBD_KO_COUNT_MIN,
 				DRBD_KO_COUNT_MAX);
@@ -212,6 +223,10 @@ void range_check(const enum range_checks what, const char *name,
 		m_strtoll_range(value, 1, name, DRBD_DEGR_WFC_TIMEOUT_MIN,
 				DRBD_DEGR_WFC_TIMEOUT_MAX);
 		break;
+	case R_OUTDATED_WFC_TIMEOUT:
+		m_strtoll_range(value, 1, name, DRBD_OUTDATED_WFC_TIMEOUT_MIN,
+				DRBD_OUTDATED_WFC_TIMEOUT_MAX);
+		break;
 	}
 }
 
@@ -257,6 +272,131 @@ static void pperror(struct d_host_info *host, struct d_proxy_info *proxy, char *
 		h_ = hosts_; }); \
 	 	h_; h_ = h_->next)
 
+/*
+ * for check_uniq: check uniqueness of
+ * resource names, ip:port, node:disk and node:device combinations
+ * as well as resource:section ...
+ * hash table to test for uniqness of these values...
+ *  256  (max minors)
+ *  *(
+ *       2 (host sections) * 4 (res ip:port node:disk node:device)
+ *     + 4 (other sections)
+ *     + some more,
+ *       if we want to check for scoped uniqueness of *every* option
+ *   )
+ *     since nobody (?) will actually use more than a dozend minors,
+ *     this should be more than enough.
+ */
+struct hsearch_data global_htable;
+void check_uniq_init(void)
+{
+	memset(&global_htable, 0, sizeof(global_htable));
+	if (!hcreate_r(256 * ((2 * 4) + 4), &global_htable)) {
+		fprintf(stderr, "Insufficient memory.\n");
+		exit(E_exec_error);
+	};
+}
+
+/* some settings need only be unique within one resource definition.
+ * we need currently about 8 + (number of host) * 8 entries,
+ * 200 should be much more than enough. */
+struct hsearch_data per_resource_htable;
+void check_upr_init(void)
+{
+	static int created = 0;
+	if (config_valid >= 2)
+		return;
+	if (created)
+		hdestroy_r(&per_resource_htable);
+	memset(&per_resource_htable, 0, sizeof(per_resource_htable));
+	if (!hcreate_r(256, &per_resource_htable)) {
+		fprintf(stderr, "Insufficient memory.\n");
+		exit(E_exec_error);
+	};
+	created = 1;
+}
+
+/* FIXME
+ * strictly speaking we don't need to check for uniqueness of disk and device names,
+ * but for uniqueness of their major:minor numbers ;-)
+ */
+int vcheck_uniq(struct hsearch_data *ht, const char *what, const char *fmt, va_list ap)
+{
+	int rv;
+	ENTRY e, *ep;
+	e.key = e.data = ep = NULL;
+
+	/* if we are done parsing the config file,
+	 * switch off this paranoia */
+	if (config_valid >= 2)
+		return 1;
+
+	rv = vasprintf(&e.key, fmt, ap);
+
+	if (rv < 0) {
+		perror("vasprintf");
+		exit(E_thinko);
+	}
+
+	if (EXIT_ON_CONFLICT && !what) {
+		fprintf(stderr, "Oops, unset argument in %s:%d.\n", __FILE__,
+			__LINE__);
+		exit(E_thinko);
+	}
+	m_asprintf((char **)&e.data, "%s:%u", config_file, fline);
+	hsearch_r(e, FIND, &ep, ht);
+	//fprintf(stderr, "FIND %s: %p\n", e.key, ep);
+	if (ep) {
+		if (what) {
+			fprintf(stderr,
+				"%s: conflicting use of %s '%s' ...\n"
+				"%s: %s '%s' first used here.\n",
+				(char *)e.data,  what, ep->key,
+				(char *)ep->data, what, ep->key);
+		}
+		free(e.key);
+		free(e.data);
+		config_valid = 0;
+	} else {
+		//fprintf(stderr, "ENTER %s\t=>\t%s\n", e.key, (char *)e.data);
+		hsearch_r(e, ENTER, &ep, ht);
+		if (!ep) {
+			fprintf(stderr, "hash table entry (%s => %s) failed\n",
+					e.key, (char *)e.data);
+			exit(E_thinko);
+		}
+		ep = NULL;
+	}
+	if (EXIT_ON_CONFLICT && ep)
+		exit(E_config_invalid);
+	return !ep;
+}
+
+int check_uniq(const char *what, const char *fmt, ...)
+{
+	int rv;
+	va_list ap;
+
+	va_start(ap, fmt);
+	rv = vcheck_uniq(&global_htable, what, fmt, ap);
+	va_end(ap);
+
+	return rv;
+}
+
+/* unique per resource */
+int check_upr(const char *what, const char *fmt, ...)
+{
+	int rv;
+	va_list ap;
+
+	va_start(ap, fmt);
+	rv = vcheck_uniq(&per_resource_htable, what, fmt, ap);
+	va_end(ap);
+
+	return rv;
+}
+
 void check_meta_disk(struct d_host_info *host)
 {
 	struct d_name *h;
@@ -367,7 +507,9 @@ static void parse_global(void)
 	}
 	EXP('{');
 	while (1) {
-		switch (yylex()) {
+		int token = yylex();
+		fline = line;
+		switch (token) {
 		case TK_DISABLE_IP_VERIFICATION:
 			global_options.disable_ip_verification = 1;
 			break;
@@ -427,10 +569,12 @@ static struct d_option *parse_options_d(int token_switch, int token_option,
 	enum range_checks rc;
 
 	struct d_option *options = NULL, *ro = NULL;
+	c_section_start = line;
 	fline = line;
 
 	while (1) {
 		token = yylex();
+		fline = line;
 		if (token == token_switch) {
 			options = APPEND(options, new_opt(yylval.txt, NULL));
 		} else if (token == token_option) {
@@ -467,38 +611,53 @@ static struct d_option *parse_options(int token_switch, int token_option)
 	return parse_options_d(token_switch, token_option, 0, NULL, NULL);
 }
 
-static void parse_address(struct d_name *on_hosts, char** addr, char** port, char** af)
+static void __parse_address(char** addr, char** port, char** af)
 {
-	struct d_name *h;
 	switch(yylex()) {
-	case TK_SCI:
+	case TK_SCI:   /* 'ssocks' was names 'sci' before. */
+		if (af)
+			*af = strdup("ssocks");
+		EXP(TK_IPADDR);
+		break;
+	case TK_SSOCKS:
 	case TK_IPV4:
-		*af = yylval.txt;
+		if (af)
+			*af = yylval.txt;
 		EXP(TK_IPADDR);
 		break;
 	case TK_IPV6:
-		*af = yylval.txt;
+		if (af)
+			*af = yylval.txt;
 		EXP('[');
 		EXP(TK_IPADDR6);
 		break;
 	case TK_IPADDR:
-		*af = strdup("ipv4");
+		if (af)
+			*af = strdup("ipv4");
 		break;
 	/* case '[': // Do not foster people's laziness ;)
 		EXP(TK_IPADDR6);
 		*af = strdup("ipv6");
 		break; */
 	default:
-		pe_expected("sci | ipv4 | ipv6 | <ipv4 address> ");
+		pe_expected("ssocks | ipv4 | ipv6 | <ipv4 address> ");
 	}
 
-	*addr = yylval.txt;
-	if (!strcmp(*af, "ipv6"))
+	if (addr)
+		*addr = yylval.txt;
+	if (af && !strcmp(*af, "ipv6"))
 		EXP(']');
 	EXP(':');
 	EXP(TK_INTEGER);
-	*port = yylval.txt;
+	if (port)
+		*port = yylval.txt;
 	range_check(R_PORT, "port", yylval.txt);
+}
+
+static void parse_address(struct d_name *on_hosts, char** addr, char** port, char** af)
+{
+	struct d_name *h;
+	__parse_address(addr, port, af);
 	if (!strcmp(*addr, "127.0.0.1") || !strcmp(*addr, "::1"))
 		for_each_host(h, on_hosts)
 			check_uniq("IP", "%s:%s:%s", h->name, *addr, *port);
@@ -588,11 +747,104 @@ static void parse_meta_disk(char **disk, char** index)
 	}
 }
 
+static void check_minor_nonsense(const char *devname, const int explicit_minor)
+{
+	if (!devname)
+		return;
+
+	/* if devname is set, it starts with /dev/drbd */
+	if (only_digits(devname + 9)) {
+		int m = strtol(devname + 9, NULL, 10);
+		if (m == explicit_minor)
+			return;
+
+		fprintf(stderr,
+			"%s:%d: explicit minor number must match with device name\n"
+			"\tTry \"device /dev/drbd%u minor %u;\",\n"
+			"\tor leave off either device name or explicit minor.\n"
+			"\tArbitrary device names must start with /dev/drbd_\n"
+			"\tmind the '_'! (/dev/ is optional, but drbd_ is required)\n",
+			config_file, fline, explicit_minor, explicit_minor);
+		config_valid = 0;
+		return;
+	} else if (devname[9] == '_')
+		return;
+
+	fprintf(stderr,
+		"%s:%d: arbitrary device name must start with /dev/drbd_\n"
+		"\tmind the '_'! (/dev/ is optional, but drbd_ is required)\n",
+		config_file, fline);
+	config_valid = 0;
+	return;
+}
+
+static void parse_device(struct d_name* on_hosts, int *minor, char **device)
+{
+	struct d_name *h;
+	int m;
+
+	switch (yylex()) {
+	case TK_STRING:
+		if (!strncmp("drbd", yylval.txt, 4)) {
+			m_asprintf(device, "/dev/%s", yylval.txt);
+			free(yylval.txt);
+		} else
+			*device = yylval.txt;
+
+		if (strncmp("/dev/drbd", *device, 9)) {
+			fprintf(stderr,
+				"%s:%d: device name must start with /dev/drbd\n"
+				"\t(/dev/ is optional, but drbd is required)\n",
+				config_file, fline);
+			config_valid = 0;
+			/* no goto out yet,
+			 * as that would additionally throw a parse error */
+		}
+		switch (yylex()) {
+		default:
+			pe_expected("minor | ;");
+			/* fall through */
+		case ';':
+			m = dt_minor_of_dev(*device);
+			if (m < 0) {
+				fprintf(stderr,
+					"%s:%d: no minor given nor device name conains a minor number\n",
+					config_file, fline);
+				config_valid = 0;
+			}
+			*minor = m;
+			goto out;
+		case TK_MINOR:
+			; /* double fall through */
+		}
+	case TK_MINOR:
+		EXP(TK_INTEGER);
+		*minor = atoi(yylval.txt);
+		EXP(';');
+
+		/* if both device name and minor number are explicitly given,
+		 * force /dev/drbd<minor-number> or /dev/drbd_<arbitrary> */
+		check_minor_nonsense(*device, *minor);
+	}
+out:
+	for_each_host(h, on_hosts) {
+		check_uniq("device-minor", "device-minor:%s:%u", h->name, *minor);
+		check_uniq("device", "device:%s:%s", h->name, *device);
+	}
+}
+
+enum parse_host_section_flags {
+	REQUIRE_ALL = 1,
+	BY_ADDRESS  = 2,
+};
+
 static void parse_host_section(struct d_resource *res,
-			       struct d_name* on_hosts, int require_all)
+			       struct d_name* on_hosts,
+			       enum parse_host_section_flags flags)
 {
 	struct d_host_info *host;
 	struct d_name *h;
+	int in_braces = 1;
 
 	c_section_start = line;
 	fline = line;
@@ -600,15 +852,46 @@ static void parse_host_section(struct d_resource *res,
 	host=calloc(1,sizeof(struct d_host_info));
 	host->on_hosts = on_hosts;
 	host->config_line = c_section_start;
+	host->device_minor = -1;
+
+	if (flags & BY_ADDRESS) {
+		/* floating <address> {} */
+		char *fake_uname = NULL;
+		int token;
+
+		host->by_address = 1;
+		__parse_address(&host->address, &host->port, &host->address_family);
+		check_uniq("IP", "%s:%s", host->address, host->port);
+		if (!strcmp(host->address_family, "ipv6"))
+			m_asprintf(&fake_uname, "ipv6 [%s]:%s", host->address, host->port);
+		else
+			m_asprintf(&fake_uname, "%s:%s", host->address, host->port);
+		on_hosts = names_from_str(fake_uname);
+		host->on_hosts = on_hosts;
+
+		token = yylex();
+		switch(token) {
+		case '{':
+			break;
+		case ';':
+			in_braces = 0;
+			break;
+		default:
+			pe_expected_got("{ | ;", token);
+		}
+	}
+
 	for_each_host(h, on_hosts)
-		check_uniq("host section", "%s: on %s", res->name, h->name);
+		check_upr("host section", "%s: on %s", res->name, h->name);
 	res->all_hosts = APPEND(res->all_hosts, host);
 
-	while (1) {
-		switch (yylex()) {
+	while (in_braces) {
+		int token = yylex();
+		fline = line;
+		switch (token) {
 		case TK_DISK:
 			for_each_host(h, on_hosts)
-				check_uniq("disk statement", "%s:%s:disk", res->name, h->name);
+				check_upr("disk statement", "%s:%s:disk", res->name, h->name);
 			EXP(TK_STRING);
 			host->disk = yylval.txt;
 			for_each_host(h, on_hosts)
@@ -617,62 +900,50 @@ static void parse_host_section(struct d_resource *res,
 			break;
 		case TK_DEVICE:
 			for_each_host(h, on_hosts)
-				check_uniq("device statement", "%s:%s:device", res->name, h->name);
-			EXP(TK_STRING);
-			host->device = yylval.txt;
-			for_each_host(h, on_hosts)
-				check_uniq("device", "device:%s:%s", h->name, yylval.txt);
-			if (dt_minor_of_dev(host->device) < 0) {
-				fprintf(stderr, "%s:%d: cannot determine minor number of %s\n",
-					config_file, line, host->device);
-				config_valid = 0;
-			}
-			EXP(';');
+				check_upr("device statement", "%s:%s:device", res->name, h->name);
+			parse_device(on_hosts, &host->device_minor, &host->device);
 			break;
 		case TK_ADDRESS:
+			if (host->by_address) {
+				fprintf(stderr,
+					"%s:%d: address statement not allowed for floating {} host sections\n",
+					config_file, fline);
+				config_valid = 0;
+				exit(E_config_invalid);
+			}
 			for_each_host(h, on_hosts)
-				check_uniq("address statement", "%s:%s:address", res->name, h->name);
+				check_upr("address statement", "%s:%s:address", res->name, h->name);
 			parse_address(on_hosts, &host->address, &host->port, &host->address_family);
 			range_check(R_PORT, "port", host->port);
 			break;
 		case TK_META_DISK:
 			for_each_host(h, on_hosts)
-				check_uniq("meta-disk statement", "%s:%s:meta-disk", res->name, h->name);
+				check_upr("meta-disk statement", "%s:%s:meta-disk", res->name, h->name);
 			parse_meta_disk(&host->meta_disk, &host->meta_index);
 			check_meta_disk(host);
 			break;
 		case TK_FLEX_META_DISK:
 			for_each_host(h, on_hosts)
-				check_uniq("meta-disk statement", "%s:%s:meta-disk", res->name, h->name);
+				check_upr("meta-disk statement", "%s:%s:meta-disk", res->name, h->name);
 			EXP(TK_STRING);
 			host->meta_disk = yylval.txt;
 			if (strcmp("internal", yylval.txt)) {
 				host->meta_index = strdup("flexible");
 			}
 			check_meta_disk(host);
-			switch (yylex()) {
-			case TK__MAJOR:
-				EXP(TK_INTEGER);
-				host->meta_major = atoi(yylval.txt);
-				EXP(TK__MINOR);
-				EXP(TK_INTEGER);
-				host->meta_minor = atoi(yylval.txt);
-				EXP(';');
-			case ';':
-				break;
-			}
+			EXP(';');
 			break;
 		case TK_PROXY:
 			parse_proxy_section(host);
 			break;
 		case '}':
-			goto break_loop;
+			in_braces = 0;
+			break;
 		default:
 			pe_expected("disk | device | address | meta-disk "
 				    "| flexible-meta-disk");
 		}
 	}
-      break_loop:
 
 	/* Inherit device, disk, meta_disk and meta_index from the resource. */
 	if(!host->disk && res->disk) {
@@ -683,8 +954,12 @@ static void parse_host_section(struct d_resource *res,
 
 	if(!host->device && res->device) {
 		host->device = strdup(res->device);
+	}
+
+	if (host->device_minor == -1 && res->device_minor != -1) {
+		host->device_minor = res->device_minor;
 		for_each_host(h, on_hosts)
-			check_uniq("device", "device:%s:%s", h->name, host->device);
+			check_uniq("device-minor", "device-minor:%s:%d", h->name, host->device_minor);
 	}
 
 	if(!host->meta_disk && res->meta_disk) {
@@ -693,9 +968,9 @@ static void parse_host_section(struct d_resource *res,
 		check_meta_disk(host);
 	}
 
-	if (!require_all)
+	if (!(flags & REQUIRE_ALL))
 		return;
-	if (!host->device)
+	if (!host->device && host->device_minor == -1)
 		derror(host, res, "device");
 	if (!host->disk)
 		derror(host, res, "disk");
@@ -747,58 +1022,33 @@ void parse_skip()
 static void parse_stacked_section(struct d_resource* res)
 {
 	struct d_host_info *host;
-	struct d_resource *l_res, *tmp;
 	struct d_name *h;
-	char *l_res_name;
 
 	c_section_start = line;
 	fline = line;
 
 	host=calloc(1,sizeof(struct d_host_info));
+	host->device_minor = -1;
 	res->all_hosts = APPEND(res->all_hosts, host);
 	EXP(TK_STRING);
-	l_res_name = yylval.txt;
-	check_uniq("stacked-on-top-of", "stacked:%s", l_res_name);
-
-	for_each_resource(l_res, tmp, config) {
-		if (!strcmp(l_res->name, l_res_name))
-			break;
-	}
-	if (l_res == NULL) {
-		fprintf(stderr, "%s:%d: in resource %s, "
-			"referenced resource '%s' not yet defined.\n",
-			config_file, c_section_start, res->name, l_res_name);
-		exit(E_config_invalid);
-	}
-	if (l_res->stacked) {
-		fprintf(stderr,
-			"%s:%d: in resource %s, stacked-on-top-of %s { ... }:\n"
-			"\tFIXME. I won't stack stacked resources.\n",
-			config_file, c_section_start, res->name, l_res_name);
-		exit(E_config_invalid);
-	}
-
-	host->on_hosts = concat_names(l_res->me->on_hosts, l_res->peer->on_hosts);
-	host->lower = l_res;
+	check_uniq("stacked-on-top-of", "stacked:%s", yylval.txt);
+	host->lower_name = yylval.txt;
 
 	m_asprintf(&host->meta_disk, "%s", "internal");
 	m_asprintf(&host->meta_index, "%s", "internal");
-	m_asprintf(&host->disk, "%s", l_res->me->device);
 
 	EXP('{');
 	while (1) {
 		switch(yylex()) {
 		case TK_DEVICE:
-			EXP(TK_STRING);
-			host->device = yylval.txt;
 			for_each_host(h, host->on_hosts)
-				check_uniq("device", "device:%s:%s", h->name, yylval.txt);
-			EXP(';');
+				check_upr("device statement", "%s:%s:device", res->name, h->name);
+			parse_device(host->on_hosts, &host->device_minor, &host->device);
 			break;
 		case TK_ADDRESS:
 			for_each_host(h, host->on_hosts)
-				check_uniq("address statement", "%s:%s:address", res->name, h->name);
-			parse_address(l_res->me->on_hosts, &host->address, &host->port, &host->address_family);
+				check_upr("address statement", "%s:%s:address", res->name, h->name);
+			parse_address(NULL, &host->address, &host->port, &host->address_family);
 			range_check(R_PORT, "port", yylval.txt);
 			break;
 		case TK_PROXY:
@@ -812,6 +1062,8 @@ static void parse_stacked_section(struct d_resource* res)
 	}
  break_loop:
 
+	res->stacked_on_one = 1;
+
 	/* inherit device */
 	if (!host->device && res->device) {
 		host->device = strdup(res->device);
@@ -819,10 +1071,14 @@ static void parse_stacked_section(struct d_resource* res)
 			check_uniq("device", "device:%s:%s", h->name, host->device);
 	}
 
-	if (!host->device)
-		derror(host,res,"device");
-	if (!host->disk)
-		derror(host,res,"disk");
+	if (host->device_minor == -1 && res->device_minor != -1) {
+		host->device_minor = res->device_minor;
+		for_each_host(h, host->on_hosts)
+			check_uniq("device-minor", "device-minor:%s:%d", h->name, host->device_minor);
+	}
+
+	if (!host->device && host->device_minor == -1)
+		derror(host, res, "device");
 	if (!host->address)
 		derror(host,res,"address");
 	if (!host->meta_disk)
@@ -842,32 +1098,261 @@ void startup_delegate(void *ctx)
 		pe_expected("<an option keyword> | become-primary-on | stacked-timeouts");
 }
 
+void net_delegate(void *ctx)
+{
+	enum pr_flags flags = (enum pr_flags)ctx;
+
+	if (!strcmp(yytext, "discard-my-data") && flags & IgnDiscardMyData)
+		EXP(';');
+	else
+		pe_expected("an option keyword");
+}
+
+void set_me_in_resource(struct d_resource* res, int match_on_proxy)
+{
+	struct d_host_info *host;
+
+	/* Determin the local host section */
+	for (host = res->all_hosts; host; host=host->next) {
+		/* do we match  this host? */
+		if (match_on_proxy) {
+		       if (!host->proxy || !name_in_names(nodeinfo.nodename, host->proxy->on_hosts))
+			       continue;
+		} else if (host->by_address) {
+			if (!have_ip(host->address_family, host->address) &&
+				/* for debugging only, e.g. __DRBD_NODE__=10.0.0.1 */
+			    strcmp(nodeinfo.nodename, host->address))
+				continue;
+		} else if (host->lower) {
+			if (!host->lower->me)
+				continue;
+		} else {
+			if (!name_in_names(nodeinfo.nodename, host->on_hosts) &&
+			    strcmp("_this_host", host->on_hosts->name))
+				continue;
+		}
+		/* we matched. */
+		if (res->ignore) {
+			config_valid = 0;
+			fprintf(stderr,
+				"%s:%d: in resource %s, %s %s { ... }:\n"
+				"\tYou cannot ignore and define at the same time.\n",
+				res->config_file, host->config_line, res->name,
+				host->lower ? "stacked-on-top-of" : "on",
+				host->lower ? host->lower->name : names_to_str(host->on_hosts));
+		}
+		if (res->me) {
+			config_valid = 0;
+			fprintf(stderr,
+				"%s:%d: in resource %s, %s %s { ... } ... %s %s { ... }:\n"
+				"\tThere are multiple host sections for this node.\n",
+				res->config_file, host->config_line, res->name,
+				res->me->lower ? "stacked-on-top-of" : "on",
+				res->me->lower ? res->me->lower->name : names_to_str(res->me->on_hosts),
+				host->lower ? "stacked-on-top-of" : "on",
+				host->lower ? host->lower->name : names_to_str(host->on_hosts));
+		}
+		res->me = host;
+		if (host->lower)
+			res->stacked = 1;
+	}
+
+	/* If there is no me, implicitly ignore that resource */
+	if (!res->me) {
+		res->ignore = 1;
+		return;
+	}
+}
+
+void set_peer_in_resource(struct d_resource* res, int peer_required)
+{
+	struct d_host_info *host = NULL;
+
+	if (res->ignore)
+		return;
+
+	/* me must be already set */
+	if (!res->me) {
+		/* should have been implicitly ignored. */
+		fprintf(stderr, "%s:%d: in resource %s:\n"
+				"\tcannot determin the peer, don't even know myself!\n",
+				res->config_file, res->start_line, res->name);
+		exit(E_thinko);
+	}
+
+	/* only one host section? */
+	if (!res->all_hosts->next) {
+		if (peer_required) {
+			fprintf(stderr,
+				"%s:%d: in resource %s:\n"
+				"\tMissing section 'on <PEER> { ... }'.\n",
+				res->config_file, res->start_line, res->name);
+			config_valid = 0;
+		}
+		return;
+	}
+
+	/* short cut for exactly two host sections.
+	 * silently ignore any --peer connect_to_host option. */
+	if (res->all_hosts->next->next == NULL) {
+		res->peer = res->all_hosts == res->me ?
+			res->all_hosts->next : res->all_hosts;
+		if (dry_run > 1 && connect_to_host)
+			fprintf(stderr,
+				"%s:%d: in resource %s:\n"
+				"\tIgnoring --peer '%s': there are only two host sections.\n",
+				res->config_file, res->start_line, res->name, connect_to_host);
+		return;
+	}
+
+	/* Multiple peer hosts to choose from.
+	 * we need some help! */
+	if (!connect_to_host) {
+		if (peer_required) {
+			fprintf(stderr,
+				"%s:%d: in resource %s:\n"
+				"\tThere are multiple host sections for the peer node.\n"
+				"\tUse the --peer option to select which peer section to use.\n",
+				res->config_file, res->start_line, res->name);
+			config_valid = 0;
+		}
+		return;
+	}
+
+	for (host = res->all_hosts; host; host=host->next) {
+		if (host->by_address && strcmp(connect_to_host, host->address))
+			continue;
+		if (host->proxy && !name_in_names(nodeinfo.nodename, host->proxy->on_hosts))
+			continue;
+		if (!name_in_names(connect_to_host, host->on_hosts))
+			continue;
+
+		if (host == res->me) {
+			fprintf(stderr,
+				"%s:%d: in resource %s\n"
+				"\tInvoked with --peer '%s', but that matches myself!\n",
+				res->config_file, res->start_line, res->name, connect_to_host);
+			res->peer = NULL;
+			break;
+		}
+
+		if (res->peer) {
+			fprintf(stderr,
+				"%s:%d: in resource %s:\n"
+				"\tInvoked with --peer '%s', but that matches multiple times!\n",
+				res->config_file, res->start_line, res->name, connect_to_host);
+			res->peer = NULL;
+			break;
+		}
+		res->peer = host;
+	}
+
+	if (peer_required && !res->peer) {
+		config_valid = 0;
+		if (!host)
+			fprintf(stderr,
+				"%s:%d: in resource %s:\n"
+				"\tNo host ('on' or 'floating') section matches --peer '%s'\n",
+				res->config_file, res->start_line, res->name, connect_to_host);
+	}
+}
+
+void set_on_hosts_in_res(struct d_resource *res)
+{
+	struct d_resource *l_res, *tmp;
+	struct d_host_info *host, *host2;
+	struct d_name *h, **last;
+
+	for (host = res->all_hosts; host; host=host->next) {
+		if (host->lower_name) {
+			for_each_resource(l_res, tmp, config) {
+				if (!strcmp(l_res->name, host->lower_name))
+					break;
+			}
+
+			if (l_res == NULL) {
+				fprintf(stderr, "%s:%d: in resource %s, "
+					"referenced resource '%s' not defined.\n",
+					res->config_file, res->start_line, res->name, l_res->name);
+				config_valid = 0;
+			}
+
+			/* Simple: host->on_hosts = concat_names(l_res->me->on_hosts, l_res->peer->on_hosts); */
+			last = NULL;
+			for (host2 = l_res->all_hosts; host2; host2 = host2->next)
+				if (!host2->lower_name)
+					append_names(&host->on_hosts, &last, host2->on_hosts);
+
+			host->lower = l_res;
+
+			/* */
+			if (!strcmp(host->address, "127.0.0.1") || !strcmp(host->address, "::1"))
+				for_each_host(h, host->on_hosts)
+					check_uniq("IP", "%s:%s:%s", h->name, host->address, host->port);
+
+		}
+	}
+}
+
+void set_disk_in_res(struct d_resource *res)
+{
+	struct d_host_info *host;
+
+	if (res->ignore)
+		return;
+
+	for (host = res->all_hosts; host; host=host->next) {
+		if (host->lower) {
+			if (res->stacked && host->lower->stacked) {
+				fprintf(stderr,
+					"%s:%d: in resource %s, stacked-on-top-of %s { ... }:\n"
+					"\tFIXME. I won't stack stacked resources.\n",
+					res->config_file, res->start_line, res->name, host->lower_name);
+				config_valid = 0;
+			}
+
+			if (host->lower->ignore)
+				continue;
+
+			if (host->lower->me->device)
+				m_asprintf(&host->disk, "%s", host->lower->me->device);
+			else
+				m_asprintf(&host->disk, "/dev/drbd%u", host->lower->me->device_minor);
+
+			if (!host->disk)
+				derror(host,res,"disk");
+		}
+	}
+}
+
 struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 {
 	struct d_resource* res;
-	struct d_host_info *host;
 	struct d_name *host_names;
 	int token;
 
-	fline = line;
-
+	check_upr_init();
 	check_uniq("resource section", res_name);
 
 	res=calloc(1,sizeof(struct d_resource));
 	res->name = res_name;
-	res->me_minor = -1; /* will be set once in dt_minor_of_res */
+	res->device_minor = -1;
+	res->config_file = config_file;
+	res->start_line = line;
 
 	while(1) {
-		switch((token=yylex())) {
+		token = yylex();
+		fline = line;
+		switch(token) {
 		case TK_PROTOCOL:
-			check_uniq("protocol statement","%s: protocol",res->name);
+			check_upr("protocol statement","%s: protocol",res->name);
 			EXP(TK_STRING);
 			res->protocol=yylval.txt;
 			EXP(';');
 			break;
 		case TK_ON:
 			parse_hosts(&host_names, '{');
-			parse_host_section(res, host_names, 1);
+			parse_host_section(res, host_names, REQUIRE_ALL);
 			break;
 		case TK_STACKED:
 			parse_stacked_section(res);
@@ -896,6 +1381,9 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 			host_names = names_from_str("_remote_host");
 			parse_host_section(res, host_names, 0);
 			break;
+		case TK_FLOATING:
+			parse_host_section(res, NULL, REQUIRE_ALL + BY_ADDRESS);
+			break;
 		case TK_DISK:
 			switch (token=yylex()) {
 			case TK_STRING:
@@ -903,7 +1391,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 				EXP(';');
 				break;
 			case '{':
-				check_uniq("disk section", "%s:disk", res->name);
+				check_upr("disk section", "%s:disk", res->name);
 				res->disk_options = parse_options(TK_DISK_SWITCH,
 								  TK_DISK_OPTION);
 				break;
@@ -913,19 +1401,22 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 			}
 			break;
 		case TK_NET:
-			check_uniq("net section", "%s:net", res->name);
+			check_upr("net section", "%s:net", res->name);
 			EXP('{');
-			res->net_options = parse_options(TK_NET_SWITCH,
-							 TK_NET_OPTION);
+			res->net_options = parse_options_d(TK_NET_SWITCH,
+							   TK_NET_OPTION,
+							   TK_NET_DELEGATE,
+							   &net_delegate,
+							   (void *)flags);
 			break;
 		case TK_SYNCER:
-			check_uniq("syncer section", "%s:syncer", res->name);
+			check_upr("syncer section", "%s:syncer", res->name);
 			EXP('{');
 			res->sync_options = parse_options(TK_SYNCER_SWITCH,
 							  TK_SYNCER_OPTION);
 			break;
 		case TK_STARTUP:
-			check_uniq("startup section", "%s:startup", res->name);
+			check_upr("startup section", "%s:startup", res->name);
 			EXP('{');
 			res->startup_options=parse_options_d(TK_STARTUP_SWITCH,
 							     TK_STARTUP_OPTION,
@@ -934,20 +1425,19 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 							     res);
 			break;
 		case TK_HANDLER:
-			check_uniq("handlers section", "%s:handlers", res->name);
+			check_upr("handlers section", "%s:handlers", res->name);
 			EXP('{');
 			res->handlers =  parse_options(0, TK_HANDLER_OPTION);
 			break;
 		case TK_PROXY:
-			check_uniq("proxy section", "%s:proxy", res->name);
+			check_upr("proxy section", "%s:proxy", res->name);
 			EXP('{');
 			res->proxy_options =  parse_options(TK_PROXY_SWITCH,
 							    TK_PROXY_OPTION);
 			break;
 		case TK_DEVICE:
-			EXP(TK_STRING);
-			res->device = yylval.txt;
-			EXP(';');
+			check_upr("device statement", "%s:device", res->name);
+			parse_device(NULL, &res->device_minor, &res->device);
 			break;
 		case TK_META_DISK:
 			parse_meta_disk(&res->meta_disk, &res->meta_index);
@@ -972,87 +1462,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 
  exit_loop:
 
-	/* Determin the local host section and the peer host section. */
-	host = res->all_hosts;
-	while (host) {
-		if (!res->ignore && (res->me && res->peer)) {
-			fprintf(stderr,
-				"%s:%d: in resource %s, "
-				"unsupported third host section %s %s { ... }.\n",
-				config_file, host->config_line, res->name,
-				host->lower ? "stacked-on-top-of" : "on",
-				host->lower ? host->lower->name : names_to_str(host->on_hosts));
-			exit(E_config_invalid);
-		}
-
-		if (name_in_names(nodeinfo.nodename, host->on_hosts) ||
-		    name_in_names("_this_host", host->on_hosts) ||
-		    ( host->proxy && name_in_names(nodeinfo.nodename, host->proxy->on_hosts))) {
-			if (res->ignore) {
-				config_valid = 0;
-				fprintf(stderr,
-					"%s:%d: in resource %s, %s %s { ... }:\n"
-					"\tYou cannot ignore and define at the same time.\n",
-					config_file, host->config_line, res->name,
-					host->lower ? "stacked-on-top-of" : "on",
-					host->lower ? host->lower->name : names_to_str(host->on_hosts));
-			}
-			if (res->me) {
-				config_valid = 0;
-				fprintf(stderr,
-					"%s:%d: in resource %s, %s %s { ... } ... %s %s { ... }:\n"
-					"\tThere are multiple host sections for this node.\n",
-					config_file, host->config_line, res->name,
-					res->me->lower ? "stacked-on-top-of" : "on",
-					res->me->lower ? res->me->lower->name : names_to_str(res->me->on_hosts),
-					host->lower ? "stacked-on-top-of" : "on",
-					host->lower ? host->lower->name : names_to_str(host->on_hosts));
-			}
-			res->me = host;
-			if (host->lower)
-				res->stacked = 1;
-		} else {
-			/* This needs to be refined as soon as we support
-			   multiple peers in the resource section */
-			if (res->peer) {
-				if (!res->me) {
-					/* just store it anyways into ->me */
-					/* reorder, so it dumps out in the same order as read in */
-					res->me = res->peer;
-					res->peer = host;
-					res->ignore = 1; /* implicit ignore */
-				} else {
-					/* hm. if that did not work, I cannot ignore it */
-					config_valid = 0;
-					fprintf(stderr, "THINKO 1\n");
-					exit(E_thinko);
-				}
-			}
-			res->peer = host;
-		}
-		host=host->next;
-	}
-
-	if(flags & ThisHRequired && !res->me) {
-		config_valid = 0;
-
-		fprintf(stderr,
-			"%s:%d: in resource %s, there is no host section"
-			" for this host.\n"
-			"\tMissing 'on %s {...}' ?\n",
-			config_file, c_section_start, res->name,
-			nodeinfo.nodename);
-	}
-	if(flags & PeerHRequired && !res->peer) {
-		config_valid = 0;
-
-		fprintf(stderr,
-			"%s:%d: in resource %s, there is no host section"
-			" for the peer host.\n"
-			"\tMissing 'on <peer-name> {...}' ?\n",
-			config_file, c_section_start, res->name);
-	}
-	if(flags == NoneHAllowed && ( res->me || res->peer ) ) {
+	if (flags == NoneHAllowed && res->all_hosts) {
 		config_valid = 0;
 
 		fprintf(stderr,
@@ -1064,13 +1474,86 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 	return res;
 }
 
+void post_parse(struct d_resource *config, enum pp_flags flags)
+{
+	struct d_resource *res,*tmp;
+
+	for_each_resource(res, tmp, config)
+		if (res->stacked_on_one)
+			set_on_hosts_in_res(res); /* sets on_hosts and host->lower */
+
+	/* Needs "on_hosts" and host->lower already set */
+	for_each_resource(res, tmp, config)
+		if (!res->stacked_on_one)
+			set_me_in_resource(res, flags & match_on_proxy);
+
+	/* Needs host->lower->me already set */
+	for_each_resource(res, tmp, config)
+		if (res->stacked_on_one)
+			set_me_in_resource(res, flags & match_on_proxy);
+
+	// Needs "me" set already
+	for_each_resource(res, tmp, config)
+		if (res->stacked_on_one)
+			set_disk_in_res(res);
+}
+
+void include_file(FILE *f, char* name)
+{
+	int saved_line;
+	char* saved_config_file;
+
+	saved_line = line;
+	saved_config_file = config_file;
+	line = 1;
+	config_file = name;
+
+	my_yypush_buffer_state(f);
+	my_parse();
+	yypop_buffer_state();
+
+	line = saved_line;
+	config_file = saved_config_file;
+}
+
+void include_stmt(char *str)
+{
+	glob_t glob_buf;
+	FILE *f;
+	size_t i;
+
+	f = fopen(str, "r");
+	if (f) {
+		include_file(f, str);
+		fclose(f);
+	} else if (glob(str, 0, NULL, &glob_buf) == 0) {
+		for (i=0; i<glob_buf.gl_pathc; i++) {
+			f = fopen(glob_buf.gl_pathv[i], "r");
+			if (f)
+				include_file(f, strdup(glob_buf.gl_pathv[i]));
+			fclose(f);
+		}
+		globfree(&glob_buf);
+	} else {
+		fprintf(stderr,
+			"%s:%d: Failed to open include file '%s'.\n",
+			config_file, line, yylval.txt);
+		config_valid = 0;
+	}
+}
+
 void my_parse(void)
 {
-	common = NULL;
-	config = NULL;
+	static int global_htable_init = 0;
+	if (!global_htable_init) {
+		check_uniq_init();
+		global_htable_init = 1;
+	}
 
 	while (1) {
-		switch (yylex()) {
+		int token = yylex();
+		fline = line;
+		switch(token) {
 		case TK_GLOBAL:
 			parse_global();
 			break;
@@ -1081,16 +1564,20 @@ void my_parse(void)
 		case TK_RESOURCE:
 			EXP(TK_STRING);
 			EXP('{');
-			config=APPEND(config,
-				      parse_resource(yylval.txt,BothHRequired));
+			config = APPEND(config, parse_resource(yylval.txt, 0));
 			break;
 		case TK_SKIP:
 			parse_skip();
 			break;
+		case TK_INCLUDE:
+			EXP(TK_STRING);
+			EXP(';');
+			include_stmt(yylval.txt);
+			break;
 		case 0:
 			return;
 		default:
-			pe_expected("global | common | resource | skip");
+			pe_expected("global | common | resource | skip | include");
 		}
 	}
 }
