@@ -36,6 +36,7 @@ enum range_checks
 	R_MAX_BUFFERS,
 	R_MAX_EPOCH_SIZE,
 	R_SNDBUF_SIZE,
+	R_RCVBUF_SIZE,
 	R_KO_COUNT,
 	R_RATE,
 	R_GROUP,
@@ -44,6 +45,7 @@ enum range_checks
 	R_META_IDX,
 	R_WFC_TIMEOUT,
 	R_DEGR_WFC_TIMEOUT,
+	R_OUTDATED_WFC_TIMEOUT,
 };
 
 enum yytokentype {
@@ -64,6 +66,7 @@ enum yytokentype {
 	TK_COMMON,
 	TK_ADDRESS,
 	TK_DEVICE,
+	TK_MINOR,
 	TK_META_DISK,
 	TK_FLEX_META_DISK,
 	TK_MINOR_COUNT,
@@ -86,8 +89,6 @@ enum yytokentype {
 	TK_YES,
 	TK_NO,
 	TK__IS_DEFAULT,
-	TK__MAJOR,
-	TK__MINOR,
 	TK__THIS_HOST,
 	TK__REMOTE_HOST,
 	TK_PROXY,
@@ -100,9 +101,13 @@ enum yytokentype {
 	TK_ERR_DQSTRING_TOO_LONG,
 	TK_ERR_DQSTRING,
 	TK_SCI,
+	TK_SSOCKS,
 	TK_IPV4,
 	TK_IPV6,
 	TK_IPADDR6,
+	TK_NET_DELEGATE,
+	TK_INCLUDE,
+	TK_FLOATING,
 };
 
 typedef struct YYSTYPE {
@@ -119,3 +124,5 @@ extern char* yytext;
 
 /* avoid compiler warnings about implicit declaration */
 int yylex(void);
+void my_yypush_buffer_state(FILE *f);
+void yypop_buffer_state (void );
