@@ -62,8 +62,10 @@ YYSTYPE yylval;
 
 /* int     force = 0; now extern, see drbdtool_common.c */
 int	verbose = 0;
+int	ignore_sanity_checks = 0;
 
 struct option metaopt[] = {
+    { "ignore-sanity-checks",  no_argument, &ignore_sanity_checks, 1000 },
     { "force",  no_argument,    0, 'f' },
     { "verbose",  no_argument,    0, 'v' },
     { NULL,     0,              0, 0 },
@@ -205,7 +207,7 @@ struct md_cpu {
 	u32 al_nr_extents;	/* important for restoring the AL */
 	s32 bm_offset;		/* signed sector offset to the bitmap, from here */
 	/* Since DRBD 0.8 we have uuid instead of gc */
-	u64 uuid[UUID_SIZE];
+	u64 uuid[UI_SIZE];
 	u32 flags;
 	u64 device_uuid;
 	u32 bm_bytes_per_bit;
@@ -271,7 +273,7 @@ struct format_ops {
  * -- DRBD 0.6 --------------------------------------
  */
 
-struct __attribute__ ((packed)) md_on_disk_06 {
+struct __packed md_on_disk_06 {
 	be_u32 gc[GEN_CNT_SIZE];	/* generation counter */
 	be_u32 magic;
 };
@@ -308,7 +310,7 @@ int v06_validate_md(struct format *cfg)
  * -- DRBD 0.7 --------------------------------------
  */
 
-struct __attribute__ ((packed)) md_on_disk_07 {
+struct __packed md_on_disk_07 {
 	be_u64 la_kb;		/* last agreed size. */
 	be_u32 gc[GEN_CNT_SIZE];	/* generation counter */
 	be_u32 magic;
@@ -424,20 +426,20 @@ int is_valid_md(int f,
  * these stay the same for 0.8, too:
  */
 
-struct __attribute__ ((packed)) al_sector_cpu {
+struct __packed al_sector_cpu {
 	u32 magic;
 	u32 tr_number;
-	struct __attribute__ ((packed)) {
+	struct __packed {
 		u32 pos;
 		u32 extent;
 	} updates[62];
 	u32 xor_sum;
 };
 
-struct __attribute__ ((packed)) al_sector_on_disk {
+struct __packed al_sector_on_disk {
 	be_u32 magic;
 	be_u32 tr_number;
-	struct __attribute__ ((packed)) {
+	struct __packed {
 		be_u32 pos;
 		be_u32 extent;
 	} updates[62];
@@ -448,9 +450,9 @@ struct __attribute__ ((packed)) al_sector_on_disk {
  * -- DRBD 0.8 --------------------------------------
  */
 
-struct __attribute__ ((packed)) md_on_disk_08 {
+struct __packed md_on_disk_08 {
 	be_u64 la_sect;		/* last agreed size. */
-	be_u64 uuid[UUID_SIZE];   // UUIDs.
+	be_u64 uuid[UI_SIZE];   // UUIDs.
 	be_u64 device_uuid;
 	be_u64 reserved_u64_1;
 	be_u32 flags;
@@ -462,7 +464,7 @@ struct __attribute__ ((packed)) md_on_disk_08 {
 	be_u32 bm_bytes_per_bit;
 	be_u32 reserved_u32[4];
 
-	char reserved[8 * 512 - (8*(UUID_SIZE+3)+4*11)];
+	char reserved[8 * 512 - (8*(UI_SIZE+3)+4*11)];
 };
 
 void md_disk_08_to_cpu(struct md_cpu *cpu, const struct md_on_disk_08 *disk)
@@ -471,7 +473,7 @@ void md_disk_08_to_cpu(struct md_cpu *cpu, const struct md_on_disk_08 *disk)
 
 	memset(cpu, 0, sizeof(*cpu));
 	cpu->la_sect = be64_to_cpu(disk->la_sect.be);
-	for ( i=Current ; i<UUID_SIZE ; i++ )
+	for ( i=UI_CURRENT ; i<UI_SIZE ; i++ )
 		cpu->uuid[i] = be64_to_cpu(disk->uuid[i].be);
 	cpu->device_uuid = be64_to_cpu(disk->device_uuid.be);
 	cpu->flags = be32_to_cpu(disk->flags.be);
@@ -487,7 +489,7 @@ void md_cpu_to_disk_08(struct md_on_disk_08 *disk, const struct md_cpu *cpu)
 {
 	int i;
 	disk->la_sect.be = cpu_to_be64(cpu->la_sect);
-	for ( i=Current ; i<UUID_SIZE ; i++ ) {
+	for ( i=UI_CURRENT ; i<UI_SIZE ; i++ ) {
 		disk->uuid[i].be = cpu_to_be64(cpu->uuid[i]);
 	}
 	disk->device_uuid.be = cpu_to_be64(cpu->device_uuid);
@@ -802,14 +804,14 @@ void m_set_gc(struct md_cpu *md, char **argv, int argc __attribute((unused)))
 	str = &argv[0];
 
 	do {
-		if (!m_strsep_bit(str, &md->gc[Flags], MDF_Consistent)) break;
+		if (!m_strsep_bit(str, &md->gc[Flags], MDF_CONSISTENT)) break;
 		if (!m_strsep_u32(str, &md->gc[HumanCnt])) break;
 		if (!m_strsep_u32(str, &md->gc[TimeoutCnt])) break;
 		if (!m_strsep_u32(str, &md->gc[ConnectedCnt])) break;
 		if (!m_strsep_u32(str, &md->gc[ArbitraryCnt])) break;
-		if (!m_strsep_bit(str, &md->gc[Flags], MDF_PrimaryInd)) break;
-		if (!m_strsep_bit(str, &md->gc[Flags], MDF_ConnectedInd)) break;
-		if (!m_strsep_bit(str, &md->gc[Flags], MDF_FullSync)) break;
+		if (!m_strsep_bit(str, &md->gc[Flags], MDF_PRIMARY_IND)) break;
+		if (!m_strsep_bit(str, &md->gc[Flags], MDF_CONNECTED_IND)) break;
+		if (!m_strsep_bit(str, &md->gc[Flags], MDF_FULL_SYNC)) break;
 	} while (0);
 }
 
@@ -821,15 +823,16 @@ void m_set_uuid(struct md_cpu *md, char **argv, int argc __attribute((unused)))
 	str = &argv[0];
 
 	do {
-		for ( i=Current ; i<UUID_SIZE ; i++ ) {
+		for ( i=UI_CURRENT ; i<UI_SIZE ; i++ ) {
 			if (!m_strsep_u64(str, &md->uuid[i])) return;
 		}
-		if (!m_strsep_bit(str, &md->flags, MDF_Consistent)) break;
-		if (!m_strsep_bit(str, &md->flags, MDF_WasUpToDate)) break;
-		if (!m_strsep_bit(str, &md->flags, MDF_PrimaryInd)) break;
-		if (!m_strsep_bit(str, &md->flags, MDF_ConnectedInd)) break;
-		if (!m_strsep_bit(str, &md->flags, MDF_FullSync)) break;
-		if (!m_strsep_bit(str, &md->flags, MDF_PeerOutDated)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_CONSISTENT)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_WAS_UP_TO_DATE)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_PRIMARY_IND)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_CONNECTED_IND)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_FULL_SYNC)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_PEER_OUT_DATED)) break;
+		if (!m_strsep_bit(str, &md->flags, MDF_CRASHED_PRIMARY)) break;
 	} while (0);
 }
 
@@ -842,28 +845,28 @@ int m_outdate_gc(struct md_cpu *md __attribute((unused)))
 
 int m_outdate_uuid(struct md_cpu *md)
 {
-	if ( !(md->flags & MDF_Consistent) ) {
+	if ( !(md->flags & MDF_CONSISTENT) ) {
 		return 5;
 	}
 
-	md->flags &= ~MDF_WasUpToDate;
+	md->flags &= ~MDF_WAS_UP_TO_DATE;
 
 	return 0;
 }
 
 int m_invalidate_gc(struct md_cpu *md)
 {
-	md->gc[Flags] &= ~MDF_Consistent;
-	md->gc[Flags] |= MDF_FullSync;
+	md->gc[Flags] &= ~MDF_CONSISTENT;
+	md->gc[Flags] |= MDF_FULL_SYNC;
 
 	return 5;
 }
 
 int m_invalidate_uuid(struct md_cpu *md)
 {
-	md->flags &= ~MDF_Consistent;
-	md->flags &= ~MDF_WasUpToDate;
-	md->flags |= MDF_FullSync;
+	md->flags &= ~MDF_CONSISTENT;
+	md->flags &= ~MDF_WAS_UP_TO_DATE;
+	md->flags |= MDF_FULL_SYNC;
 
 	return 0;
 }
@@ -1036,7 +1039,7 @@ int md_initialize_common(struct format *cfg, int do_disk_writes)
 		return 0;
 
 	/* do you want to initilize al to something more usefull? */
-	printf("initialising activity log\n");
+	printf("initializing activity log\n");
 	if (MD_AL_MAX_SECT_07*512 > buffer_size) {
 		fprintf(stderr, "%s:%u: LOGIC BUG\n" , __FILE__ , __LINE__ );
 		exit(111);
@@ -1060,7 +1063,7 @@ int md_initialize_common(struct format *cfg, int do_disk_writes)
 		unsigned int percent_done = 0;
 		unsigned int percent_last_report = 0;
 		size_t chunk;
-		fprintf(stderr,"initialising bitmap (%u KB)\n",
+		fprintf(stderr,"initializing bitmap (%u KB)\n",
 			(unsigned int)(bm_bytes>>10));
 
 		memset(on_disk_buffer, 0xff, buffer_size);
@@ -1138,7 +1141,7 @@ void printf_bm(struct format *cfg)
 {
 	off_t bm_on_disk_off = cfg->bm_offset;
 	le_u64 const *bm = on_disk_buffer;
-	le_u64 cw; /* current word for rll encoding */
+	le_u64 cw; /* current word for rl encoding */
 	const unsigned int n = cfg->bm_bytes/sizeof(*bm);
 	unsigned int count = 0;
 	unsigned int bits_set = 0;
@@ -1163,14 +1166,15 @@ void printf_bm(struct format *cfg)
 			i = 0;
 			n_buffer = chunk/sizeof(*bm);
 		}
+next:
 		ASSERT(i < n_buffer);
 		if (count == 0) cw = bm[i];
 		if ((i & 3) == 0) {
 			if (!count) printf_bm_eol(r);
 
-			for (j = i+1; j < n_buffer; j++) {
-				if(cw.le != bm[j].le) break;
-			}
+			/* j = i, because it may be continuation after buffer wrap */
+			for (j = i; j < n_buffer && cw.le == bm[j].le; j++)
+				;
 			j &= ~3; // round down to a multiple of 4
 			unsigned int tmp = (j-i);
 			if (tmp > 4) {
@@ -1184,13 +1188,17 @@ void printf_bm(struct format *cfg)
 				       count, le64_to_cpu(cw.le));
 				bits_set += count * generic_hweight64(cw.le);
 				count = 0;
-				continue;
+				if (r >= n)
+					break;
+				/* don't "continue;", we may have not advanced i after buffer wrap,
+				 * so that would be treated as an other buffer wrap */
+				goto next;
 			}
 		}
 		ASSERT(i < n_buffer);
 		printf(" 0x"X64(016)";", le64_to_cpu(bm[i].le));
-		r++; i++;
 		bits_set += generic_hweight64(bm[i].le);
+		r++; i++;
 	}
 	printf("\n}\n");
 	cfg->bits_set = bits_set;
@@ -1416,9 +1424,9 @@ int _v08_md_initialize(struct format *cfg, int do_disk_writes)
 	memset(&cfg->md, 0, sizeof(cfg->md));
 
 	cfg->md.la_sect = 0;
-	cfg->md.uuid[Current] = UUID_JUST_CREATED;
-	cfg->md.uuid[Bitmap] = 0;
-	for ( i=History_start ; i<=History_end ; i++ ) {
+	cfg->md.uuid[UI_CURRENT] = UUID_JUST_CREATED;
+	cfg->md.uuid[UI_BITMAP] = 0;
+	for ( i=UI_HISTORY_START ; i<=UI_HISTORY_END ; i++ ) {
 		cfg->md.uuid[i]=0;
 	}
 	cfg->md.flags = 0;
@@ -1486,8 +1494,8 @@ int meta_dstate(struct format *cfg, char **argv __attribute((unused)), int argc)
 	if (cfg->ops->open(cfg))
 		return -1;
 
-	if(cfg->md.flags & MDF_Consistent) {
-		if(cfg->md.flags & MDF_WasUpToDate) {
+	if(cfg->md.flags & MDF_CONSISTENT) {
+		if(cfg->md.flags & MDF_WAS_UP_TO_DATE) {
 			printf("Consistent/DUnknown\n");
 		} else {
 			printf("Outdated/DUnknown\n");
@@ -1581,7 +1589,7 @@ int meta_dump_md(struct format *cfg, char **argv __attribute((unused)), int argc
 		printf("\n}\n");
 	} else { // >= 08
 		printf("uuid {\n   ");
-		for ( i=Current ; i<UUID_SIZE ; i++ ) {
+		for ( i=UI_CURRENT ; i<UI_SIZE ; i++ ) {
 			printf(" 0x"X64(016)";", cfg->md.uuid[i]);
 		}
 		printf("\n");
@@ -1718,7 +1726,7 @@ int verify_dumpfile_or_restore(struct format *cfg, char **argv, int argc, int pa
 
 			ASSERT(!is_v06(cfg));
 		}
-		fprintf(stderr, "reinitialising\n");
+		fprintf(stderr, "reinitializing\n");
 		if (is_v07(cfg))
 			_v07_md_initialize(cfg,0);
 		else
@@ -1741,7 +1749,7 @@ int verify_dumpfile_or_restore(struct format *cfg, char **argv, int argc, int pa
 		EXP('}');
 	} else { // >= 08
 		EXP(TK_UUID); EXP('{');
-		for ( i=Current ; i<UUID_SIZE ; i++ ) {
+		for ( i=UI_CURRENT ; i<UI_SIZE ; i++ ) {
 			EXP(TK_U64); EXP(';');
 			cfg->md.uuid[i] = yylval.u64;
 		}
@@ -1778,7 +1786,7 @@ int verify_dumpfile_or_restore(struct format *cfg, char **argv, int argc, int pa
 			 * or the maintenance nightmare of code duplication */
 			if (parse_only) break;
 			bm[i].le = cpu_to_le64(yylval.u64);
-			if (++i == buffer_size/sizeof(*bm)) {
+			if ((unsigned)++i == buffer_size/sizeof(*bm)) {
 				pwrite_or_die(cfg->md_fd, on_disk_buffer,
 					buffer_size, bm_on_disk_off,
 					"meta_restore_md:TK_U64");
@@ -1795,7 +1803,7 @@ int verify_dumpfile_or_restore(struct format *cfg, char **argv, int argc, int pa
 			value.le = cpu_to_le64(yylval.u64);
 			while(times--) {
 				bm[i] = value;
-				if (++i == buffer_size/sizeof(*bm)) {
+				if ((unsigned)++i == buffer_size/sizeof(*bm)) {
 					pwrite_or_die(cfg->md_fd, on_disk_buffer,
 						buffer_size, bm_on_disk_off,
 						"meta_restore_md:TK_NUM");
@@ -1860,13 +1868,13 @@ void md_convert_07_to_08(struct format *cfg)
 	int i,j;
 	/*
 	 * FIXME
-	 * what about the Bitmap, and the Activity Log?
+	 * what about the UI_BITMAP, and the Activity Log?
 	 * how to bring them over for internal meta data?
 	 *
 	 * maybe just refuse to convert anything that is not
 	 * "clean"? how to detect that?
 	 *
-	 * FIXME: if I am a crashed Primary, or Inconsistent,
+	 * FIXME: if I am a crashed R_PRIMARY, or D_INCONSISTENT,
 	 * or Want-Full-Sync or the like,
 	 * refuse, and indicate how to solve this */
 
@@ -1885,17 +1893,17 @@ void md_convert_07_to_08(struct format *cfg)
 	// The MDF Flags are (nearly) the same in 07 and 08
 	cfg->md.flags = cfg->md.gc[Flags];
 
-	cfg->md.uuid[Current] =
+	cfg->md.uuid[UI_CURRENT] =
 		(u64)(cfg->md.gc[HumanCnt] & 0xffff) << 48 |
 		(u64)(cfg->md.gc[TimeoutCnt] & 0xffff) << 32 |
 		(u64)((cfg->md.gc[ConnectedCnt]+cfg->md.gc[ArbitraryCnt])
 		       & 0xffff) << 16 |
 		(u64)0xbabe;
-	cfg->md.uuid[Bitmap] = (u64)0;
+	cfg->md.uuid[UI_BITMAP] = (u64)0;
 
-	for (i = cfg->bits_set ? Bitmap : History_start, j = 1;
-		i <= History_end ; i++, j++)
-		cfg->md.uuid[i] = cfg->md.uuid[Current] - j*0x10000;
+	for (i = cfg->bits_set ? UI_BITMAP : UI_HISTORY_START, j = 1;
+		i <= UI_HISTORY_END ; i++, j++)
+		cfg->md.uuid[i] = cfg->md.uuid[UI_CURRENT] - j*0x10000;
 
 	/* unconditionally re-initialize offsets,
 	 * not necessary if fixed size external,
@@ -1912,13 +1920,13 @@ void md_convert_08_to_07(struct format *cfg)
 {
 	/*
 	 * FIXME
-	 * what about the Bitmap, and the Activity Log?
+	 * what about the UI_BITMAP, and the Activity Log?
 	 * how to bring them over for internal meta data?
 	 *
 	 * maybe just refuse to convert anything that is not
 	 * "clean"? how to detect that?
 	 *
-	 * FIXME: if I am a crashed Primary, or Inconsistent,
+	 * FIXME: if I am a crashed R_PRIMARY, or D_INCONSISTENT,
 	 * or Want-Full-Sync or the like,
 	 * refuse, and indicate how to solve this */
 
@@ -2020,7 +2028,7 @@ int may_be_jfs(const char *data, struct fstype_s *f)
 /* really large block size,
  * will always refuse */
 #define REFUSE_BSIZE 0xFFFFffffFFFF0000LLU
-#define REFUSE_IT    f->bnum = 1; f->bsize = REFUSE_BSIZE;
+#define REFUSE_IT()	do { f->bnum = 1; f->bsize = REFUSE_BSIZE; } while(0)
 int may_be_swap(const char *data, struct fstype_s *f)
 {
 	int looks_like_swap =
@@ -2030,17 +2038,65 @@ int may_be_swap(const char *data, struct fstype_s *f)
 		strncmp(data+(1<<13)-10, "SWAPSPACE2", 10) == 0;
 	if (looks_like_swap) {
 		f->type = "swap space signature";
-		REFUSE_IT
+		REFUSE_IT();
 		return 1;
 	}
 	return 0;
 }
 
-int may_be_LVM(const char *data, struct fstype_s *f)
+int guessed_size_from_pvs(struct fstype_s *f, char *dev_name)
+{
+	char buf[200];
+	size_t c;
+	unsigned long long bnum;
+	int pipes[2];
+	pid_t pid;
+
+	if (pipe(pipes))
+		return 0;
+
+	pid = fork();
+	if (pid < 0)
+		return 0;
+
+	if (pid == 0) {
+		/* child */
+		char *argv[] = {
+			"pvs", "--noheadings", "--nosuffix", "--units", "s",
+			"-o", "pv_size",
+			dev_name,
+			NULL,
+		};
+		close(pipes[0]);
+		dup2(pipes[1],1);
+		close(pipes[1]);
+		close(2);
+		execvp(argv[0], argv);
+		_exit(0);
+	}
+	/* parent */
+	close(pipes[1]);
+	c = read(pipes[0], buf, sizeof(buf)-1);
+	close(pipes[0]);
+
+	if (c <= 0)
+		return 0;
+
+	buf[c] = 0;
+	if (1 == sscanf(buf, " %llu\n", &bnum)) {
+		f->bnum = bnum;
+		f->bsize = 512;
+		return 1;
+	} else
+		return 0;
+}
+
+int may_be_LVM(const char *data, struct fstype_s *f, char *dev_name)
 {
 	if (strncmp("LVM2",data+0x218,4) == 0) {
 		f->type = "LVM2 physical volume signature";
-		REFUSE_IT
+		if (!guessed_size_from_pvs(f, dev_name))
+			REFUSE_IT();
 		return 1;
 	}
 	return 0;
@@ -2050,6 +2106,8 @@ void check_for_existing_data(struct format *cfg)
 {
 	struct fstype_s f;
 	size_t i;
+	u64 fs_kB;
+	u64 max_usable_kB;
 
 	PREAD(cfg->md_fd, on_disk_buffer, SO_MUCH, 0);
 
@@ -2068,7 +2126,7 @@ void check_for_existing_data(struct format *cfg)
 
 	(void)(
 	may_be_swap     (on_disk_buffer,&f) ||
-	may_be_LVM      (on_disk_buffer,&f) ||
+	may_be_LVM      (on_disk_buffer,&f, cfg->md_device_name) ||
 
 	may_be_extX     (on_disk_buffer,&f) ||
 	may_be_xfs      (on_disk_buffer,&f) ||
@@ -2088,6 +2146,30 @@ void check_for_existing_data(struct format *cfg)
 	printf("bm_offset %llu\n", (long long unsigned)cfg->bm_offset);
 
 	printf("\nFound %s ", f.type);
+
+	/* FIXME overflow check missing!
+	 * relevant for ln2(bsize) + ln2(bnum) >= 64, thus only for
+	 * device sizes of more than several exa byte.
+	 * seems irrelevant to me for now.
+	 */
+	fs_kB = ((f.bsize * f.bnum) + (1<<10)-1) >> 10;
+#define min(x,y) ((x) < (y) ? (x) : (y))
+	max_usable_kB =
+		min( cfg->md_offset,
+		min( cfg->al_offset,
+		     cfg->bm_offset )) >> 10;
+#undef min
+
+	if (ignore_sanity_checks) {
+		if (f.bsize != REFUSE_BSIZE)
+			printf("which uses "U64" kB\n", fs_kB);
+		else
+			printf("\n");
+		printf("current configuration leaves usable "U64" kB\n", max_usable_kB);
+		printf("\nIgnoring sanity check on user request.\n\n");
+		return;
+	}
+
 	if (f.bnum) {
 		if (cfg->md_index >= 0 ||
 		    cfg->md_index == DRBD_MD_INDEX_FLEX_EXT) {
@@ -2100,14 +2182,6 @@ void check_for_existing_data(struct format *cfg)
 			exit(40); /* FIXME sane exit code! */
 		}
 
-		/* FIXME overflow check missing!
-		 * relevant for ln2(bsize) + ln2(bnum) >= 64, thus only for
-		 * device sizes of more than several exa byte.
-		 * seems irrelevant to me for now.
-		 */
-		u64 fs_kB = ((f.bsize * f.bnum) + (1<<10)-1) >> 10;
-		u64 max_usable_kB;
-
 		if (f.bsize == REFUSE_BSIZE) {
 			printf(
 "\nDevice size would be truncated, which\n"
@@ -2120,13 +2194,6 @@ void check_for_existing_data(struct format *cfg)
 			exit(40); /* FIXME sane exit code! */
 		}
 
-#define min(x,y) ((x) < (y) ? (x) : (y))
-		max_usable_kB =
-			min( cfg->md_offset,
-			min( cfg->al_offset,
-			     cfg->bm_offset )) >> 10;
-#undef min
-
 		/* looks like file system data */
 		printf("which uses "U64" kB\n", fs_kB);
 		printf("current configuration leaves usable "U64" kB\n", max_usable_kB);
@@ -2141,10 +2208,15 @@ void check_for_existing_data(struct format *cfg)
 "   * zero out the device (destroy the filesystem)\n"
 "Operation refused.\n\n");
 			exit(40); /* FIXME sane exit code! */
+		} else {
+			printf(
+"\nEven though it looks like this would place the new meta data into\n"
+"unused space, you still need to confirm, as this is only a guess.\n");
 		}
-	}
-	if (!confirmed(" ==> This might destroy existing data! <==\n\n"
-			"Do you want to proceed?")) {
+	} else
+		printf("\n ==> This might destroy existing data! <==\n");
+
+	if (!confirmed("Do you want to proceed?")) {
 		printf("Operation cancelled.\n");
 		exit(1); // 1 to avoid online resource counting
 	}
@@ -2374,7 +2446,7 @@ int meta_create_md(struct format *cfg, char **argv __attribute((unused)), int ar
 	 * to flexible size, we'd need to move the AL and bitmap
 	 * over to the new location!
 	 * But the upgrade procedure in such case is documented to first get
-	 * the previous DRBD into "clean" Connected Secondary/Secondary, so AL
+	 * the previous DRBD into "clean" C_CONNECTED R_SECONDARY/R_SECONDARY, so AL
 	 * and bitmap should be empty anyways.
 	 */
 	err = err || cfg->ops->md_cpu_to_disk(cfg); // <- short circuit
@@ -2382,7 +2454,7 @@ int meta_create_md(struct format *cfg, char **argv __attribute((unused)), int ar
 	if (err)
 		fprintf(stderr, "operation failed\n");
 	else
-		printf("New drbd meta data block sucessfully created.\n");
+		printf("New drbd meta data block successfully created.\n");
 
 	return err;
 }
@@ -2670,6 +2742,8 @@ int main(int argc, char **argv)
 		break;
 
 	    switch (c) {
+	    case 0:
+		break;
 	    case 'f':
 		force = 1;
 		break;
