@@ -61,23 +61,27 @@ char* ppsize(char* buf, size_t size)
 	return buf;
 }
 
-const char* make_optstring(struct option *options,char startc)
+const char *make_optstring(struct option *options, char startc)
 {
-  static char buffer[200];
-  struct option *opt;
-  char *c;
+	static char buffer[200];
+	struct option *opt;
+	char *c;
 
-  opt = options;
-  c=buffer;
-  if(startc) *c++=startc;
-  while(opt->name)
-    {
-      *c++=opt->val;
-      if(opt->has_arg) *c++=':';
-      opt++;
-    }
-  *c=0;
-  return buffer;
+	opt = options;
+	c = buffer;
+	if (startc)
+		*c++ = startc;
+	while (opt->name) {
+		if (0 < opt->val || opt->val < 256) {
+			*c++ = opt->val;
+			if (opt->has_arg)
+				*c++ = ':';
+		}
+		opt++;
+	}
+
+	*c = 0;
+	return buffer;
 }
 
 int
@@ -246,26 +250,59 @@ int get_fd_lockfile_timeout(const char *path, int seconds)
 int dt_minor_of_dev(const char *device)
 {
 	struct stat sb;
+	long m;
+	int digits_only = only_digits(device);
+	const char *c = device;
 
-	if(stat(device,&sb)) {
-		// On udev/devfs based system the device nodes does not
-		// exist before the module is loaded. Therefore assume that
-		// the number in the device name is the minor number.
-		const char *c;
+	/* On udev/devfs based system the device nodes does not
+	 * exist before the drbd is created.
+	 *
+	 * If the device name starts with /dev/drbd followed by
+	 * only digits, or if only digits are given,
+	 * those digits are the minor number.
+	 *
+	 * Otherwise, we cannot reliably determine the minor number!
+	 *
+	 * We allow "arbitrary" device names in drbd.conf,
+	 * and those may well contain digits.
+	 * Interpreting any digits as minor number is dangerous!
+	 */
+	if (!digits_only) {
+		if (!strncmp("/dev/drbd", device, 9) &&
+		    only_digits(device + 9))
+			c = device + 9;
 
-		c=device;
-		while(*c) {
-			if(isdigit(*c))
-				return strtol(c,NULL,10);
-			c++;
-		}
-		/* if there is not even a digit in that name, fail. */
-		return -1;
+		/* if the device node exists,
+		 * and is a block device with the correct major,
+		 * do not enforce further naming conventions.
+		 * people without udev, and not using drbdadm
+		 * may do whatever they like. */
+		else if (!stat(device,&sb) &&
+			 S_ISBLK(sb.st_mode) &&
+			 major(sb.st_rdev) == LANANA_DRBD_MAJOR)
+			return minor(sb.st_rdev);
+
+		else
+			return -1;
 	}
 
-	return minor(sb.st_rdev);
+	/* ^[0-9]+$ or ^/dev/drbd[0-9]+$ */
+
+	errno = 0;
+	m = strtol(c, NULL, 10);
+	if (!errno)
+		return m;
+
+	return -1;
 }
 
+int only_digits(const char *s)
+{
+	const char *c;
+	for (c = s; isdigit(*c); c++)
+		;
+	return c != s && *c == 0;
+}
 
 int dt_lock_drbd(const char* device)
 {
@@ -274,8 +311,13 @@ int dt_lock_drbd(const char* device)
 	char lfname[40];
 	int dev_major,dev_minor;
 
-	dev_major = 147; //LANANA_DRBD_MAJOR;
-	if (!stat(device, &drbd_stat)) {
+	dev_major = LANANA_DRBD_MAJOR;
+
+	/* if called from drbdadm, "device" is usually just the minor number.
+	 * if someone happens to mkdir /0, drbdsetup 0 anything would simply
+	 * say "0 is not a block device!" */
+
+	if (!only_digits(device) && !stat(device, &drbd_stat)) {
 		if (!S_ISBLK(drbd_stat.st_mode)) {
 			fprintf(stderr, "%s is not a block device!\n", device);
 			exit(20);
@@ -288,28 +330,7 @@ int dt_lock_drbd(const char* device)
 					device, dev_major, LANANA_DRBD_MAJOR);
 			exit(20);
 		}
-	} else {
-		/* See also dt_minor_of_dev(),
-		 * maybe we are about to create it?
-		 * But warn people which expect drbdsetup to behave like
-		 * "drbdsetup cute_name attach", it is
-		 * "drbdsetup /dev/drbd# attach". */
-		if (0 < strncmp("/dev/drbd0",device,10) ||
-		    0 > strncmp("/dev/drbd9",device,10))
-		{
-			/* warn only, don't fail, there may be a reason to not
-			 * use this device naming convention.
-			 *
-			 * but people expecting to be able to use arbitrary
-			 * names for device names, just because we default to
-			 * drbd0, should be made aware of their error. */
-			fprintf(stderr,
-				"We expect our device names to be named\n"
-				"/dev/drbd<minor-number>, not %s\n", device);
-		}
 	}
-
-
 	/* THINK.
 	 * maybe we should also place a fcntl lock on the
 	 * _physical_device_ we open later...
@@ -321,8 +342,6 @@ int dt_lock_drbd(const char* device)
 	 * pretending it belongs to an other, you'll screw up completely.
 	 *
 	 * We should store something in the meta data to detect such abuses.
-	 * Philipp, see my suggestion for "/var/lib/drbd/drbd-toc",
-	 * or /etc/drbd/ for that matter ...
 	 */
 
 	/* NOTE that /var/lock/drbd-*-* may not be "secure",
@@ -334,7 +353,7 @@ int dt_lock_drbd(const char* device)
 	if (dev_minor < 0) {
 		fprintf(stderr,
 			"Could not determine device minor number of '%s'.\n"
-			"Try /dev/drbd<minor-number> instead.\n", device);
+			"Try /dev/drbd<minor-number> or just <minor-number> instead.\n", device);
 		exit(20);
 	}
 	snprintf(lfname,39,"/var/lock/drbd-%d-%d",dev_major,dev_minor);
@@ -354,14 +373,14 @@ void dt_unlock_drbd(int lock_fd)
 void dt_print_gc(const __u32* gen_cnt)
 {
 	printf("%d:%d:%d:%d:%d:%d:%d:%d\n",
-	       gen_cnt[Flags] & MDF_Consistent ? 1 : 0,
+	       gen_cnt[Flags] & MDF_CONSISTENT ? 1 : 0,
 	       gen_cnt[HumanCnt],
 	       gen_cnt[TimeoutCnt],
 	       gen_cnt[ConnectedCnt],
 	       gen_cnt[ArbitraryCnt],
-	       gen_cnt[Flags] & MDF_PrimaryInd ? 1 : 0,
-	       gen_cnt[Flags] & MDF_ConnectedInd ? 1 : 0,
-	       gen_cnt[Flags] & MDF_FullSync ? 1 : 0);
+	       gen_cnt[Flags] & MDF_PRIMARY_IND ? 1 : 0,
+	       gen_cnt[Flags] & MDF_CONNECTED_IND ? 1 : 0,
+	       gen_cnt[Flags] & MDF_FULL_SYNC ? 1 : 0);
 }
 
 void dt_pretty_print_gc(const __u32* gen_cnt)
@@ -378,32 +397,33 @@ void dt_pretty_print_gc(const __u32* gen_cnt)
 	       "   --------+-----+-----+-----+-----+-----+-----+-----+\n"
 	       "       %3s | %3d | %3d | %3d | %3d | %3s | %3s | %3s  \n"
 	       "\n",
-	       gen_cnt[Flags] & MDF_Consistent ? "1/c" : "0/i",
+	       gen_cnt[Flags] & MDF_CONSISTENT ? "1/c" : "0/i",
 	       gen_cnt[HumanCnt],
 	       gen_cnt[TimeoutCnt],
 	       gen_cnt[ConnectedCnt],
 	       gen_cnt[ArbitraryCnt],
-	       gen_cnt[Flags] & MDF_PrimaryInd ? "1/p" : "0/s",
-	       gen_cnt[Flags] & MDF_ConnectedInd ? "1/c" : "0/n",
-	       gen_cnt[Flags] & MDF_FullSync ? "1/y" : "0/n");
+	       gen_cnt[Flags] & MDF_PRIMARY_IND ? "1/p" : "0/s",
+	       gen_cnt[Flags] & MDF_CONNECTED_IND ? "1/c" : "0/n",
+	       gen_cnt[Flags] & MDF_FULL_SYNC ? "1/y" : "0/n");
 }
 
 void dt_print_uuids(const __u64* uuid, unsigned int flags)
 {
 	int i;
 	printf(X64(016)":"X64(016)":",
-	       uuid[Current],
-	       uuid[Bitmap]);
-	for ( i=History_start ; i<=History_end ; i++ ) {
+	       uuid[UI_CURRENT],
+	       uuid[UI_BITMAP]);
+	for ( i=UI_HISTORY_START ; i<=UI_HISTORY_END ; i++ ) {
 		printf(X64(016)":", uuid[i]);
 	}
-	printf("%d:%d:%d:%d:%d:%d\n",
-	       flags & MDF_Consistent ? 1 : 0,
-	       flags & MDF_WasUpToDate ? 1 : 0,
-	       flags & MDF_PrimaryInd ? 1 : 0,
-	       flags & MDF_ConnectedInd ? 1 : 0,
-	       flags & MDF_FullSync ? 1 : 0,
-	       flags & MDF_PeerOutDated ? 1 : 0);
+	printf("%d:%d:%d:%d:%d:%d:%d\n",
+	       flags & MDF_CONSISTENT ? 1 : 0,
+	       flags & MDF_WAS_UP_TO_DATE ? 1 : 0,
+	       flags & MDF_PRIMARY_IND ? 1 : 0,
+	       flags & MDF_CONNECTED_IND ? 1 : 0,
+	       flags & MDF_FULL_SYNC ? 1 : 0,
+	       flags & MDF_PEER_OUT_DATED ? 1 : 0,
+	       flags & MDF_CRASHED_PRIMARY ? 1 : 0);
 }
 
 void dt_pretty_print_uuids(const __u64* uuid, unsigned int flags)
@@ -417,14 +437,24 @@ void dt_pretty_print_uuids(const __u64* uuid, unsigned int flags)
 "       V               V                 V         V\n");
 	dt_print_uuids(uuid, flags);
 	printf(
-"                                                                    ^ ^ ^ ^ ^ ^\n"
-"                                      -<  Data consistancy flag  >--+ | | | | |\n"
-"                             -<  Data was/is currently up-to-date  >--+ | | | |\n"
-"                                  -<  Node was/is currently primary  >--+ | | |\n"
-"                                  -<  Node was/is currently connected  >--+ | |\n"
-"         -<  Node was in the progress of setting all bits in the bitmap  >--+ |\n"
-"                        -<  The peer's disk was out-dated or inconsistent  >--+\n"
+"                                                                    ^ ^ ^ ^ ^ ^ ^\n"
+"                                      -<  Data consistancy flag  >--+ | | | | | |\n"
+"                             -<  Data was/is currently up-to-date  >--+ | | | | |\n"
+"                                  -<  Node was/is currently primary  >--+ | | | |\n"
+"                                  -<  Node was/is currently connected  >--+ | | |\n"
+"         -<  Node was in the progress of setting all bits in the bitmap  >--+ | |\n"
+"                        -<  The peer's disk was out-dated or inconsistent  >--+ |\n"
+"      -<  This node was a crashed primary, and has not seen its peer since   >--+\n"
 "\n");
+	printf("flags:%s %s, %s, %s%s%s\n",
+	       (flags & MDF_CRASHED_PRIMARY) ? " crashed" : "",
+	       (flags & MDF_PRIMARY_IND) ? "Primary" : "Secondary",
+	       (flags & MDF_CONNECTED_IND) ? "Connected" : "StandAlone",
+	       (flags & MDF_CONSISTENT)
+			?  ((flags & MDF_WAS_UP_TO_DATE) ? "UpToDate" : "Outdated")
+			: "Inconsistent",
+	       (flags & MDF_FULL_SYNC) ? ", need full sync" : "",
+	       (flags & MDF_PEER_OUT_DATED) ? ", peer Outdated" : "");
 }
 
 /*    s: token buffer
@@ -520,7 +550,7 @@ void get_random_bytes(void* buffer, int len)
 	close(fd);
 }
 
-const char* shell_escape(char* s)
+const char* shell_escape(const char* s)
 {
 	/* ugly static buffer. so what. */
 	static char buffer[1024];
