@@ -5,6 +5,10 @@
 #include "drbd_endian.h"
 #include <stdarg.h>
 
+#ifndef __packed
+#define __packed __attribute__((packed))
+#endif
+
 #define ARRY_SIZE(A) (sizeof(A)/sizeof(A[0]))
 
 #define COMM_TIMEOUT 120
@@ -36,6 +40,7 @@ enum new_strtoll_errs {
 
 struct option;
 
+extern int only_digits(const char *s);
 extern int dt_lock_drbd(const char* device);
 extern void dt_unlock_drbd(int lock_fd);
 extern void dt_release_lockfile(int drbd_fd);
@@ -56,7 +61,7 @@ extern void get_random_bytes(void* buffer, int len);
 extern int force; /* global option to force implicit confirmation */
 extern int confirmed(const char *text);
 
-extern const char* shell_escape(char* s);
+extern const char* shell_escape(const char* s);
 
 /* Since glibc 2.8~20080505-0ubuntu7 asprintf() is declared with the
    warn_unused_result attribute.... */
